#Example_21j  Fitting volume element to image-plane data where out of plane error is ignored

fem def para;r;fitting;example;       # Declares array dimensions
fem def coord;r;rcCoord;example;      # Defines the coordinate system
fem define base;r;3Lin;example;       # Defines tri-linear Hermite basis functions

$name="cube"
fem def node;r;$name;example;         # Reads in nodal information
fem def elem;r;$name;example;         # Reads in element information
# Export original geometry to view in Cmgui
fem export node;$name as $name offset 100
fem export elem;$name as $name offset_elem 100

fem def data;r;landmark;example;      # Defines the data-points for time T0
fem def xi;c contain                  # Calculates the xi positions of the data points inside the mesh
fem def xi;w;landmark                 # Writes the xi positions to file 
fem export data;landmark as landmark offset 100

fem def data;r;target;example;        # Defines the data-points for time T1 (deformed state)
fem def xi;r;landmark                 # Read in the xi-positions to calculate error between landmark and target points
fem export data;target as target error offset 200 

fem update field from geometry        # Prepare the field variables
fem def fit;r;fitCube;example geom    # Setting the fit options
fem fit in_plane                      # execute the fit with the in_plane option, i.e. ignoring out-of-plane movement of the data-point

$nFits=5        # Set the number of fit iterations
for($n=2;$n<=$nFits;$n++)
{
  print "\n\n\************************************\n"
  print "\n\n\Fit number $n\n"
  print "\n\n\************************************\n"

  fem update node fit
  fem def data;r;landmark;example;
  $name = "newLandmark";
  fem define data;c from_xi
  fem define data;w;$name

  fem update field from geometry
  fem def data;r;newLandmark
  fem def xi;c contain
  fem def xi;w;landmark
  fem def data;r;target;example;
  fem def xi;r;landmark

  fem def fit;r;fitCube;example geom 
  fem fit in_plane
}

# Update the geometry, write it to file and export it to be viewed in CMGUI
fem update node fit 
$name = "transformed_cube"
fem def node;w;$name
fem def elem;w;$name
fem export node;$name as $name offset 000
fem export elem;$name as $name offset_elem 000

# Write the final positions of the data points to file and export them to be viewed in CMGUI
fem export data;projections as projections error offset 300 
fem def data;r;landmark;example;
$name = "finalDataPts";
fem define data;c from_xi
fem define data;w;$name
fem export data;$name as $name error offset 0


