#include <stdio.h>
#include <math.h>

/* This program creates a data file for cmiss that consists of */
/* data points spaced over a half sphere.  A flat probability  */
/* distribution is used to generate */
/*   0<theta<360 */
/*   0<phi<90 */
/*   1.9<radius<2.1 */
/* The half sphere is in the -z halfspace */

#define NUM_DATA_POINTS 400

double random01(void)
/* Return a random number in the range 0-1 */
{
  double number;
  static double divisor = 2.147484e+09;

  number = random();
  /* divide by 2**31 -1 */
  number /= divisor;
  if((number<0.0)||(number>1.0)) 
  {
    printf("Warning!  Number not in 0-1 range %lf\n",number);
  }

  return number;
}

#define RADIUS 2
#define RADIUS_SD 0.1
#define PI 3.1415927
#define PI_180 (PI/180.0)
void main()
{
  int i;
  double coordinate[3],theta,phi,radius;
  FILE *data_file;

  data_file = fopen("geometry_fit.ipdata","w");
  fprintf(data_file,"Data file\n");
  for(i=0;i<NUM_DATA_POINTS;i++)
  {
    radius = RADIUS+RADIUS_SD*(1.0-2.0*random01());
    theta = 360.0*random01();
    phi = 90.0*random01();
    /* transform to cartesian*/
    coordinate[0] = radius*sin(phi*PI_180)*cos(theta*PI_180);
    coordinate[1] = radius*sin(phi*PI_180)*sin(theta*PI_180);
    coordinate[2] = -radius*cos(phi*PI_180);
    /* output in cmiss format */
    fprintf(data_file,"%i %lf %lf %lf %lf %lf %lf\n",i+1,coordinate[0],
      coordinate[1],coordinate[2],1.0,1.0,1.0);
  }
  fclose(data_file);
}
