package MyFunc;


# readStrain
# This file reads the first strain at first Gauss point and returns
sub readStrain 
{
  my ($N) = @_;

  my $inputFile = "strain.exdata";
  my $bContinue = 1;
  my $line;
  my $strain = 999.999;
  
  if (!(-e $inputFile))
  {
    die "Input-file doesn't exist";
  }
  CORE::open(INFID, $inputFile) || die "Couldn't open $inputFile";
  while (($line = <INFID>) && ($bContinue))
  {  
    if($line =~ m/ E         1 3/)
    {
      $bContinue = 0;
      for(my $n=1;$n<=$N;$n++)
      {
        $line = <INFID>;
      }
      $strain = 1.0*$line; 
    }
  }
  # Find LV pressur from element 1.
  CORE::close(INFID);
  return($strain);
} #readStrain


sub readStress 
{
  my ($N) = @_;

  my $inputFile = "stress.exdata";
  my $bContinue = 1;
  my $line;
  my $stress = 999.999;
  
  if (!(-e $inputFile))
  {
    die "Input-file doesn't exist";
  }
  CORE::open(INFID, $inputFile) || die "Couldn't open $inputFile";
  while (($line = <INFID>) && ($bContinue))
  {  
    if($line =~ m/ E         1 3/)
    {
      $bContinue = 0;
      for(my $n=1;$n<=$N;$n++)
      {
        $line = <INFID>;
      }
      $stress = 1.0*$line; 
    }
  }
  # Find LV pressur from element 1.
  CORE::close(INFID);
  return($stress);
} #readStress


sub outputValues 
{
  my ($strain, $stress, $trueStress, $outputFile) = @_;
  
  CORE::open(OUTFID, ">> $outputFile") || die "Couldn't open $outputFile";
  print OUTFID sprintf("%10.5f   %10.5f   %10.5f\n", $strain, $stress, $trueStress);
  CORE::close(OUTFID);
} 


sub calcStress 
{
  my ($strain, $k, $a, $b) = @_;
  
  my $TOL = 0.9;
  my $T = 0;
  my $e1 = $strain;
  if ($strain < 0)
  {
    $e1 = 0;
  } elsif($strain > $TOL*$a)
  {
    $e1 = $TOL*$a;
  }

  $slope = $k/(($a-$e1)**$b)*(2 + 4*$b*$e1/($a-$e1) + $b*$e1**2/(($a-$e1)**2)*($b+1));
  #$T = $k*(2*$e1*$a+($b-2)*$e1^2)/(($a-$e1)^$b+1);
  $T = $k*(2*$e1*$a+($b-2)*$e1**2)/(($a-$e1)**($b+1));
  #print "\n\n\n$T $k  $a  $b $e1\n\n\n";
  if(($strain < 0) || ($strain > $TOL*$a))
  {
    $T = $T + $slope*($strain-$e1);
    $kk = ($strain-$e1);
    #print "\n\n\n$T $k  $a  $b $slope\n\n\n";
  }
  return($T);
} 


# stretchCube
# This file displaces some nodes in cube.ipnode
sub stretchCube 
{
  my ($example, $dx) = @_;
  
  my $inputFile = $example."cube.ipnode";
  my $outputFile = "cubeDeformed.ipnode";
  my $line;
  my $node = 0;
  my $dir = 0;
  my $dy = -$dx/(1+$dx);
  
  if (!(-e $inputFile))
  {
    die "Input-file doesn't exist";
  }
  CORE::open(INFID, $inputFile) || die "Couldn't open $inputFile";
  CORE::open(OUTFID, "> $outputFile") || die "Couldn't open $outputFile";
  while ($line = <INFID>)
  {  
    if($line =~ m/ Node number \[.+\]:(.+)/)
    {
      $node = $1;
    }
    if($line =~ m/ The Xj\((.+)\) coordinate is /)
    {
      $dir = $1;
    }
    if(($line =~ m/ The derivative wrt direction 1 is \[.+\]:(.+)/) && ($dir == 1))
    {
      $line = sprintf(" The derivative wrt direction 1 is [ 0.00000E+00]: %f\n", $1+$dx);
    }
    if(($line =~ m/ The derivative wrt direction 2 is \[.+\]:(.+)/) && ($dir == 2))
    {
      $line = sprintf(" The derivative wrt direction 2 is [ 0.00000E+00]: %f\n", $1+$dy);
    }
    if(($line =~ m/ The Xj\(1\) coordinate is \[.+\]:(.+)/) && 
      (($node==2) || ($node==4) || ($node==6) || ($node==8)))
    {
      $line = sprintf(" The Xj(1) coordinate is [ 0.10000E+01]: %f\n", $1+$dx);
    }
    if(($line =~ m/ The Xj\(2\) coordinate is \[.+\]:(.+)/) && 
      (($node==3) || ($node==4) || ($node==7) || ($node==8)))
    {
      $line = sprintf(" The Xj(2) coordinate is [ 0.10000E+01]: %f\n", $1+$dy);
    }
    print OUTFID $line;
  }
  # Find LV pressur from element 1.
  CORE::close(INFID);
  CORE::close(OUTFID);
  return();
} #stretchCube


# shearCube
# This file displaces some nodes in cube.ipnode
sub shearCube 
{
  my ($example, $delta) = @_;
  
  my $inputFile = $example."cube.ipnode";
  my $outputFile = "cubeDeformed.ipnode";
  my $line;
  my $node = 0;
  my $dir = 0;
  
  if (!(-e $inputFile))
  {
    die "Input-file doesn't exist";
  }
  CORE::open(INFID, $inputFile) || die "Couldn't open $inputFile";
  CORE::open(OUTFID, "> $outputFile") || die "Couldn't open $outputFile";
  while ($line = <INFID>)
  {  
    if($line =~ m/ Node number \[.+\]:(.+)/)
    {
      $node = $1;
    }
    if($line =~ m/ The Xj\((.+)\) coordinate is /)
    {
      $dir = $1;
    }
    if(($line =~ m/ The derivative wrt direction 3 is/) && ($dir == 1))
    {
      $line = " The derivative wrt direction 3 is [ 0.00000E+00]: $delta\n";
    }
    if(($line =~ m/ The Xj\(1\) coordinate is \[.+\]:(.+)/) && ($node>=5))
    {
      $line = sprintf(" The Xj(1) coordinate is [ 0.10000E+01]: %f\n", $1+$delta);
    }
    print OUTFID $line;
  }
  # Find LV pressur from element 1.
  CORE::close(INFID);
  CORE::close(OUTFID);
  return();
} #shearCube


1;
 
__END__
