#Example_5531  Contraction of an inflated orthotropic prolate element
#
#
# If the example path is not set, default to current directory
if (!defined $example) {
    $example = "./";
}
# Drop off the trailing / in the example path
$chopped = chop $example;
if ($chopped ne "/") {
    $example .= $chopped;
}

fem define param;r;prol;example;                 
fem define coordinate;r;profib;example;          # 3D prolate spheroidal
fem define node;r;profib;example;                # 4 nodes of previous example_5512
fem define base;r;profib;example;                # trilinear + quadratic hydr. pressure
fem define element;r;profib;example;             # single axisymmetric element
fem define fibre;r;profib;example;               # fibre angles of previous example_5512
fem define element;r;profib;example fibre;       # fibre elements
fem define equation;r;profib;example;            # 3D, finite elasticity
fem define material;r;profib_active;example;     # incomp; pole-zero law + active fibre stress
fem define active;r;profib;example;              # steady state calcium/length/tension
#                                                !  model for generation of active forces
fem define initial;r;profib_def;example;         # reads in inflated mesh (results from
#                                                !  previous example_5512 - inflated 2kPa)
fem define solve;r;profib;example;               # Newton iterations
fem evaluate reaction;
# ############################################ 
# Inflated solution is converged OK! 
# ############################################ 

$WALL = 1;
$CAVITY = 2;

# 
# Set up cavity region geometry 
fem define region;r;coupled;example;
fem define coord;r;cavity;example region $CAVITY;
fem define;add base;r;cavity;example;
fem define node;r;cavity;example region $CAVITY;
fem define element;r;cavity;example region $CAVITY;
# 
# Set up equations, material properties and initial conditions 
# for cavity region 
fem define equation;r;coupled;example region $WALL,$CAVITY lock;
fem define material;r;cavity_isovol;example region $CAVITY;
fem define initial;r;cavity;example region $CAVITY;
# 
# Define cavity-to-wall coupling and solution procedure information 
fem define coupling;r;coupled;example;
fem define solve;r;coupled;example coupled region $WALL,$CAVITY;
# 
# Transfer wall deformation from the inflation problem to 
# geometric dependent variables for the cavity region 
fem update solution coupled source_region $WALL;
# 
# Set up the reference state for the cavity region to be the deformed 
# configuration from the inflation problem.  To set up the 
# 'inflated cavity', the x-coord ('in 1' represents the 1st RC coord) 
# for the central cavity node (node 6) is set to the average of the 
# x-coords for the adjacent interface nodes (node 2 - this could 
# be a list of nodes) 
fem update solution cavity_reference average 6 in 1 node 2 region $CAVITY;
# 
# List undeformed (pre-inflation) and inflated cavity volumes 
fem list element total region $CAVITY;
fem list element deformed total region $CAVITY;
# ############################################ 
# cavity volume inflated from 40ml to 64ml. 
# ############################################ 
# 
fem define window;
fem draw lines region $WALL,$CAVITY;
fem draw lines dot def region $WALL,$CAVITY;
# 
# Check that the current coupled solution is converged (as expected!) 
fem evaluate reaction coupled;
# ############################################ 
# inflated state is a converged solution for the coupled problem! 
# ############################################ 

# 
# ISOVOLUMIC CONTRACTION 
# 
# Increment intracell calcium conc and solve for isovolumic contraction 
fem define active;r;profib_contract;example;
fem solve coupled increment 0.0;
# 
# Save deformed configurations for end ejection 
#fem define initial;w;profib_endisovolcontr region $WALL,$CAVITY;
# 
# ############################################ 
# The solution procedure hasn't theoretically converged, but 
# if you follow the progress of the solver and the final 
# constr vs unconstr resids, it's very close to converged. 
# Note that the ratio monotonically decreased, 
# which is also a sign of a stable solution process. 
# ############################################ 
# If previously solved isovolumic contraction then read contracted solution
#fem define initial;r;profib_endisovolcontr;example region $WALL,$CAVITY;
#fem define solve;r;coupled;example coupled region $WALL,$CAVITY;

# Check that the current coupled solution is converged (as expected!) 
#fem evaluate reaction coupled;

fem draw lines deformed dotted rgb=blue region $WALL,$CAVITY;
# 
# Check that cavity volume has not changed 
fem list element deformed total region $CAVITY;

# ############################################ 
# This contraction phase was indeed isovolumic! 
# (cavity volume still 64ml). 
# ############################################ 
# 
# EJECTION 
# 
# Decrease cavity 'stiffness parameter' and solve for ejection 
fem define material;r;cavity_eject;example region $CAVITY;
fem solve coupled increment 0.0;
# 
# Save deformed configurations for end ejection 
#fem define initial;w;profib_endeject region $WALL,$CAVITY;

# ############################################ 
# Ejection problem is SLOW to converge, but note that the ratio 
# monotonically decreases, indicating a stable solution. 
# May need to repeat the last solve command to get better convergence, 
# but will probably not get down to 10^(-10) tolerance. 
# ############################################ 
# If previously solved ejection then read ejected solution
#fem define initial;r;profib_endeject;example region $WALL,$CAVITY;
#fem define solve;r;coupled;example coupled region $WALL,$CAVITY;

# Check that the current coupled solution is converged (as expected!) 
#fem evaluate reaction coupled;
# 
fem draw;add lines deformed dotted region $WALL,$CAVITY;
# 
# Determine cavity volume at end-ejection. 
# NOTE: to calc cavity volume, the deformed position of the central 
# cavity node must be recalc'ed from the averaged adjacent wall nodes. 
# After doing this the current solution is meaningless, ie. best to save 
# deformed configurations for both regions before this (as above). 
fem update solution cavity_reference average 6 in 1 node 2 region $CAVITY;
fem list element deformed total region $CAVITY;
# ############################################ 
# Not a huge ejection fraction (!!!), but neverless a decrease 
# in volume from 64ml to 62ml. 
# ############################################ 
