# Example 5535

# A single axisymetric prolate LV model is used to investigate
# the local feedback mechanisms affecting the fibre 
# orientation.
# (Tissue Remodelling with Micro Structurally Based
#  Material Laws, Peter Hunter & Theo Arts 1997)
#
# Muscle Fibre Orientation
# PROBLEM_1: Sheet angles are all zero, fibre angles are zero at
#            the apex nodes, -90 at the endocardial base, and the 
#            epicardial base node fibre angle is varied from 20 to 90 degs.
# PROBLEM_2: Endocardial sheet angles are -45 epicardial are 45, 
#            Endocardial fibre angles are 65 epicardial
#            are varied from -20 to -90 degs.

$TRUE = 1
$FALSE = 0  

# Choose which problem to run
$PROBLEM_1 = $TRUE
$PROBLEM_2 = $FALSE

$MIN_ANGLE = 20
$MAX_ANGLE = 90
$INC_ANGLE = 10


system "rm plot.dat"
set echo

for ($ANGLE=$MIN_ANGLE;$ANGLE<=$MAX_ANGLE;$ANGLE+=$INC_ANGLE)
{

	print "\n\n\n\$ANGLE = $ANGLE\n";

  fem
  fem define parameters;r;profib;example
  fem define coordinate;r;profib;example #Defines 3 GLOBAL PROLATE SPHEROIDAL
  #                                      !  coordinates.
  #                                      !  Transmural interpolation is in
  #                                      !  Focus^3.Cosh(Lamda).Sinh^2(Lamda).
  fem define node;r;;example              #Focus = 37.5. Reads 4 NODES, 3 COORDS,
  #                                      !  1 VERSION per node per nj, 0
  #                                      !  DERIVATIVES for all coords. Nodal
  #                                      !  coordinates, Xj, are (lamda,mu,theta):
  #                                      !  1=(0.43,0,0);2=(0.43,120,0);
  #                                      !  3=(0.72,0,0); 4=(0.72,120,0).
  fem define base;r;;example              #Defines 3 basis functions:
  #                                      !  1) Lagrange/Hermite with 3 Xi coords,
  #                                      !     Linear Lagrange interpolant and 1
  #                                      !     Gauss point in Xi1 dirctn (theta-
  #                                      !     circumferential) but 3 in Xi2 (mu-
  #                                      !     longitudinal) and Xi3 (lamda-
  #                                      !     transmural);
  #                                      !  2) An auxiliary basis with 5 auxiliary
  #                                      !     element parameters, pressure basis
  #                                      !     same number of Gauss points as for
  #                                      !     basis function 1, and polynomial
  #                                      !     degrees of: 0 for Xi1 and Xi2 for
  #                                      !     all 5 parameters; 0 in Xi3 for
  #                                      !     parameter 1; 1 for parameter 2; 2
  #                                      !     for parameter 3; -1 for parameter
  #                                      !     4 (Xi3=0 face pressure bc); & -2
  #                                      !     for parameter 5 (Xi3=1 face
  #                                      !     pressure bc).
  #                                      !  3) 2 Xi coords with linear Lagrange
  #                                      !     interpolant, and 3 Gauss points
  #                                      !     in Xi1 and Xi2 directions.
  fem define element;r;;example           #Defines 1 prolate spheroidal element.
  set echo
  if($PROBLEM_1)
  {
    fem define fibre;r;problem_1/profib$ANGLE;example
  } elsif($PROBLEM_2)
  {
    fem define fibre;r;problem_2/profib$ANGLE;example
  }
  set echo
  #                                      #Defines fibre angle orientation wrt Xi1
  #                                      !  direction (theta) in degrees. 
  #                                      !  degrees at outer nodes 3,4.
  fem define element;r;profib;example fibre    #Defines fibre elements.
  fem define window                      #Defines window 
  fem draw lines                         #Makes line segments visible on window.
  fem define equation;r;;example          #Static 3D finite elasticity problem with
  #                                      !  geometric coords as the dependent
  #                                      !  variables. Solution is by Galerkin
  #                                      !  finite elements in a nonlinear
  #                                      !  analysis of a nonlinear equation.
  #                                      !  Incompressible material with basis 1
  #                                      !  in all elements for dep vars 1,2,3
  #                                      !  and basis 2 in all elements for dep
  #                                      !  var 4 (hydrostatic pressure).
  fem define material;r;;example          #Stresses in constitutive law are
  #                                      !  referred to body coords.  Defines a
  #                                      !  hyperelastic constitutive law for
  #                                      !  which the strain energy function, W,
  #                                      !  is a function of fibre/transverse
  #                                      !  strains that is pole-zero in strains
  #                                      !  with 16 constant constitutive law
  #                                      !  parameters. The Lagrangian strain
  #                                      !  components are referred to the
  #                                      !  undeformed fibre coordinate system
  #                                      !  where the 1-direction is aligned with
  #                                      !  fibre axis, 2-direction is the sheet
  #                                      !  axis (in the plane of the sheet,
  #                                      !  perpendicular to the fibres), and the
  #                                      !  3-direction is the sheet-normal coordinate.

  # 
  #
  # INFLATION
  #
  #

  fem define initial;r;;example           #Boundary pressure increments are entered
  #                                      !  and hydrostatic pressure is matched
  #                                      !  across boundaries with adjacent Xi3
  #                                      !  faces. Initial displacements are all
  #                                      !  zero. For dependent variable 2
  #                                      !  (mu-dirn) fix all nodes (1..4);
  #                                      !  Dependent variable 3 (theta-dirn),
  #                                      !  fix node 2. No force bcs.
  #                                      !  Dependent variable 4 (hyd press) fix
  #                                      !  element 1, parameter 4 (Xi3=0 face
  #                                      !  inside prolate) with increment
  #                                      !  magnitude 1, and parameter 5 (Xi3=1
  #                                      !  face outside prolate) with increment
  #                                      !  magnitude 0. No force bcs.
  fem define solve;r;profib;example      #  Read in solution information.
  fem solve increment 0.2 iter 8         #Solve the problem.
  #                                      !Incrementing of the ventricular
  #                                      !  pressure is neccessary to avoid
  #                                      !  unstable solutions. This depends on
  #                                      !  the constitutive law and pressure
  #                                      !  the constitutive law and pressure
  #                                      !  magnitude, as the wall is gradually
  #                                      !  stiffened.
  fem solve increment 0.3 iter 8                         
  fem solve increment 0.5        
  #                              
  #                              
  #                              
  fem draw lines def dotted      #Draw deformed mesh on window.
  fem def initial;w;profib_def
  fem eval reac
  # ############################################ 
  # Inflated solution is converged OK! 
  # ############################################ 

  # 
  #
  # COUPLING SET UP 
  #
  #
  $WALL = 1
  $CAVITY = 2
  # 
  # Set up cavity region geometry 
  fem define region;r;coupled;example 
  fem define coord;r;cavity;example region $CAVITY
  #
  fem define;add base;r;cavity;example 
  fem define node;r;cavity;example region $CAVITY
  fem define element;r;cavity;example region $CAVITY
  # 
  # Set up equations, material properties and initial conditions 
  # for cavity region 
  fem define equation;r;coupled;example region $WALL,$CAVITY lock
  fem define material;r;cavity_isovol;example region $CAVITY 
  fem define initial;r;cavity;example region $CAVITY
  # 
  # Define cavity-to-wall coupling and solution procedure information 
  fem define coupling;r;coupled;example 
  fem define solve;r;coupled;example coupled region $WALL,$CAVITY
  # 
  # Transfer wall deformation from the inflation problem to 
  # geometric dependent variables for the cavity region 
  fem update solution coupled source_region $WALL
  # 
  # Set up the reference state for the cavity region to be the deformed 
  # configuration from the inflation problem.  To set up the 
  # 'inflated cavity', the x-coord ('in 1' represents the 1st RC coord) 
  # for the central cavity node (node 6) is set to the average of the 
  # x-coords for the adjacent interface nodes (node 2 - this could 
  # be a list of nodes) 
  fem update solution cavity_reference average 6 in 1 node 2 region $CAVITY
  # 
  # List undeformed (pre-inflation) and inflated cavity volumes 
  fem list element total region $CAVITY
  fem list element deformed total region $CAVITY
  # ############################################ 
  # cavity volume inflated from 40ml to 60ml. 
  # ############################################ 
  # 
  fem draw lines region $WALL,$CAVITY
  fem draw lines dot def region $WALL,$CAVITY
  # 
  # Check that the current coupled solution is converged (as expected!) 
  fem evaluate residuals wrt geom_params coupled
  # ############################################ 
  # inflated state is a converged solution for the coupled problem! 
  # ############################################ 
 
  #
  # 
  # ISOVOLUMIC CONTRACTION 
  # 
  # 

  # Increment intracell calcium conc and solve for isovolumic contraction 
  fem define material;r;profib_active;example 
  if($PROBLEM_1)
  {
    fem define active;r;problem_1/profib_contract_Ca20;example
  } elsif($PROBLEM_2)
  {
    fem define active;r;problem_2/profib_contract_Ca10;example
  }

  fem solve coupled increment 0.0
  # ############################################ 
  # Converged solution for the coupled wall/cavity problem.
  # ############################################ 
  #fem define initial;r;profib_endisovolcontr region WALL,CAVITY 
  # 
  # Save deformed configurations for end ejection 
  fem define initial;w;profib_endisovolcontr region $WALL,$CAVITY 
  # 
  fem draw lines deformed dotted rgb=blue region $WALL,$CAVITY
  # 
  # Check that cavity volume has not changed 
  fem list element deformed total region $CAVITY
  # ############################################ 
  # This contraction phase was indeed isovolumic! 
  # ############################################ 

  # 
  #
  # POSTPROCESSING, VISUALISATION & PLOTTING
  #  
  #
  fem export nodes;heart$ANGLE as heart
  fem export elements;heart$ANGLE as heart
  fem export gauss;heart$ANGLE as gauss
  fem evaluate field;fibre_angles$ANGLE geometric gauss # get fibre & sheet angles
  fem list stress;stresses$ANGLE fibre # get stresses
  system "echo \"\# Fibre angle ${ANGLE} degrees\" >> plot.dat"
  system "awk -v angle=${ANGLE} -f ${example}extract_fibre_angles_stress.awk ${example}elements.txt fibre_angles${ANGLE}.opfiel stresses${ANGLE}.opstre >> plot.dat"
}
system "gnuplot ${example}angle_stress.gnu"
