# This file creates the nodes for the cavity elements.
# It takes all the endocardial nodes from the rc3.ipnode file
# and copies them to the new file cavity.ipnode.
# Then nodes on the x-axis are added.

use PDL;
use Math::Trig;
$inFile = "rc3.ipnode";
$outFile = "cavity.ipnode";

if (!(-e $inFile))
{
  die "Input-file doesn't exist";
}

$boolFinished = 0;
CORE::open(INFID, $inFile) || die "Couldn't open $inFile";
CORE::open(OUTFID, ">$outFile") || die "Couldn't open $outFile";
while (($line = <INFID>) && !($boolFinished))
{  
  if($line =~ / Node number \[.+\]:(.+)/)
  {
    if($1 == 18)
    {
      $boolFinished = 1;
    } else
    {
      print OUTFID $line;
    }
  }
  elsif (grep(/ The number of nodes is/, $line))
  {
    $line = " The number of nodes is [   62]:    21\n";
    print OUTFID $line;
  }
  else
  {
    print OUTFID $line;
  }
}
CORE::close(INFID);
my $coords = readNodes($inFile);
my $x1 = (at($coords,1,2)+at($coords,1,3)+at($coords,1,4)+at($coords,1,5))/4;
my $x2 = (at($coords,1,6)+at($coords,1,7)+at($coords,1,8)+at($coords,1,9))/4;
my $x3 = (at($coords,1,10)+at($coords,1,11)+at($coords,1,12)+at($coords,1,13))/4;
my $x4 = (at($coords,1,14)+at($coords,1,15)+at($coords,1,16)+at($coords,1,17))/4;

# Apex moves so much towards base during IVC + ejection that the nodes on the long
# axis must be moved up so the bottom node doesn't end up on the outside of apex
$x1 = $x2;
$x2 = ($x2+$x3)/2;
$x3 = ($x3+$x4)/2;


print OUTFID " Node number [   28]:    101\n";
print OUTFID " The number of versions for nj=1 is [1]: 1\n";
print OUTFID " The Xj(1) coordinate is [ 0.48024E+00]:   $x1\n";
print OUTFID " The derivative wrt direction 1 is [ 0.00000E+00]:   0\n";
print OUTFID " The derivative wrt direction 2 is [ 0.00000E+00]:  0\n";
print OUTFID " The derivative wrt directions 1 & 2 is [ 0.00000E+00]:   0\n";
print OUTFID " The derivative wrt direction 3 is [ 0.00000E+00]: 0\n";
print OUTFID " The derivative wrt directions 1 & 3 is [ 0.00000E+00]: 0\n";
print OUTFID " The derivative wrt directions 2 & 3 is [ 0.00000E+00]: 0\n";
print OUTFID " The derivative wrt directions 1, 2 & 3 is [ 0.00000E+00]: 0\n";
print OUTFID " The number of versions for nj=2 is [1]: 1\n";
print OUTFID " The Xj(2) coordinate is [ 0.00000E+00]:    0\n";
print OUTFID " The derivative wrt direction 1 is [ 0.00000E+00]:   0\n";
print OUTFID " The derivative wrt direction 2 is [ 0.00000E+00]:  0\n";
print OUTFID " The derivative wrt directions 1 & 2 is [ 0.00000E+00]:   0\n";
print OUTFID " The derivative wrt direction 3 is [ 0.00000E+00]: 0\n";
print OUTFID " The derivative wrt directions 1 & 3 is [ 0.00000E+00]: 0\n";
print OUTFID " The derivative wrt directions 2 & 3 is [ 0.00000E+00]: 0\n";
print OUTFID " The derivative wrt directions 1, 2 & 3 is [ 0.00000E+00]: 0\n";
print OUTFID " The number of versions for nj=3 is [1]: 1\n";
print OUTFID " The Xj(3) coordinate is [ 0.00000E+00]:    0\n";
print OUTFID " The derivative wrt direction 1 is [ 0.00000E+00]:   0\n";
print OUTFID " The derivative wrt direction 2 is [ 0.00000E+00]:  0\n";
print OUTFID " The derivative wrt directions 1 & 2 is [ 0.00000E+00]:   0\n";
print OUTFID " The derivative wrt direction 3 is [ 0.00000E+00]: 0\n";
print OUTFID " The derivative wrt directions 1 & 3 is [ 0.00000E+00]: 0\n";
print OUTFID " The derivative wrt directions 2 & 3 is [ 0.00000E+00]: 0\n";
print OUTFID " The derivative wrt directions 1, 2 & 3 is [ 0.00000E+00]: 0\n";
print OUTFID "\n";
print OUTFID " Node number [   28]:    102\n";
print OUTFID " The number of versions for nj=1 is [1]: 1\n";
print OUTFID " The Xj(1) coordinate is [ 0.48024E+00]:   $x2\n";
print OUTFID " The derivative wrt direction 1 is [ 0.00000E+00]:   0\n";
print OUTFID " The derivative wrt direction 2 is [ 0.00000E+00]:  0\n";
print OUTFID " The derivative wrt directions 1 & 2 is [ 0.00000E+00]:   0\n";
print OUTFID " The derivative wrt direction 3 is [ 0.00000E+00]: 0\n";
print OUTFID " The derivative wrt directions 1 & 3 is [ 0.00000E+00]: 0\n";
print OUTFID " The derivative wrt directions 2 & 3 is [ 0.00000E+00]: 0\n";
print OUTFID " The derivative wrt directions 1, 2 & 3 is [ 0.00000E+00]: 0\n";
print OUTFID " The number of versions for nj=2 is [1]: 1\n";
print OUTFID " The Xj(2) coordinate is [ 0.00000E+00]:    0\n";
print OUTFID " The derivative wrt direction 1 is [ 0.00000E+00]:   0\n";
print OUTFID " The derivative wrt direction 2 is [ 0.00000E+00]:  0\n";
print OUTFID " The derivative wrt directions 1 & 2 is [ 0.00000E+00]:   0\n";
print OUTFID " The derivative wrt direction 3 is [ 0.00000E+00]: 0\n";
print OUTFID " The derivative wrt directions 1 & 3 is [ 0.00000E+00]: 0\n";
print OUTFID " The derivative wrt directions 2 & 3 is [ 0.00000E+00]: 0\n";
print OUTFID " The derivative wrt directions 1, 2 & 3 is [ 0.00000E+00]: 0\n";
print OUTFID " The number of versions for nj=3 is [1]: 1\n";
print OUTFID " The Xj(3) coordinate is [ 0.00000E+00]:    0\n";
print OUTFID " The derivative wrt direction 1 is [ 0.00000E+00]:   0\n";
print OUTFID " The derivative wrt direction 2 is [ 0.00000E+00]:  0\n";
print OUTFID " The derivative wrt directions 1 & 2 is [ 0.00000E+00]:   0\n";
print OUTFID " The derivative wrt direction 3 is [ 0.00000E+00]: 0\n";
print OUTFID " The derivative wrt directions 1 & 3 is [ 0.00000E+00]: 0\n";
print OUTFID " The derivative wrt directions 2 & 3 is [ 0.00000E+00]: 0\n";
print OUTFID " The derivative wrt directions 1, 2 & 3 is [ 0.00000E+00]: 0\n";
print OUTFID "\n";
print OUTFID " Node number [   28]:    103\n";
print OUTFID " The number of versions for nj=1 is [1]: 1\n";
print OUTFID " The Xj(1) coordinate is [ 0.48024E+00]:   $x3\n";
print OUTFID " The derivative wrt direction 1 is [ 0.00000E+00]:   0\n";
print OUTFID " The derivative wrt direction 2 is [ 0.00000E+00]:  0\n";
print OUTFID " The derivative wrt directions 1 & 2 is [ 0.00000E+00]:   0\n";
print OUTFID " The derivative wrt direction 3 is [ 0.00000E+00]: 0\n";
print OUTFID " The derivative wrt directions 1 & 3 is [ 0.00000E+00]: 0\n";
print OUTFID " The derivative wrt directions 2 & 3 is [ 0.00000E+00]: 0\n";
print OUTFID " The derivative wrt directions 1, 2 & 3 is [ 0.00000E+00]: 0\n";
print OUTFID " The number of versions for nj=2 is [1]: 1\n";
print OUTFID " The Xj(2) coordinate is [ 0.00000E+00]:    0\n";
print OUTFID " The derivative wrt direction 1 is [ 0.00000E+00]:   0\n";
print OUTFID " The derivative wrt direction 2 is [ 0.00000E+00]:  0\n";
print OUTFID " The derivative wrt directions 1 & 2 is [ 0.00000E+00]:   0\n";
print OUTFID " The derivative wrt direction 3 is [ 0.00000E+00]: 0\n";
print OUTFID " The derivative wrt directions 1 & 3 is [ 0.00000E+00]: 0\n";
print OUTFID " The derivative wrt directions 2 & 3 is [ 0.00000E+00]: 0\n";
print OUTFID " The derivative wrt directions 1, 2 & 3 is [ 0.00000E+00]: 0\n";
print OUTFID " The number of versions for nj=3 is [1]: 1\n";
print OUTFID " The Xj(3) coordinate is [ 0.00000E+00]:    0\n";
print OUTFID " The derivative wrt direction 1 is [ 0.00000E+00]:   0\n";
print OUTFID " The derivative wrt direction 2 is [ 0.00000E+00]:  0\n";
print OUTFID " The derivative wrt directions 1 & 2 is [ 0.00000E+00]:   0\n";
print OUTFID " The derivative wrt direction 3 is [ 0.00000E+00]: 0\n";
print OUTFID " The derivative wrt directions 1 & 3 is [ 0.00000E+00]: 0\n";
print OUTFID " The derivative wrt directions 2 & 3 is [ 0.00000E+00]: 0\n";
print OUTFID " The derivative wrt directions 1, 2 & 3 is [ 0.00000E+00]: 0\n";
print OUTFID "\n";
print OUTFID " Node number [   28]:    104\n";
print OUTFID " The number of versions for nj=1 is [1]: 1\n";
print OUTFID " The Xj(1) coordinate is [ 0.48024E+00]:   $x4\n";
print OUTFID " The derivative wrt direction 1 is [ 0.00000E+00]:   0\n";
print OUTFID " The derivative wrt direction 2 is [ 0.00000E+00]:  0\n";
print OUTFID " The derivative wrt directions 1 & 2 is [ 0.00000E+00]:   0\n";
print OUTFID " The derivative wrt direction 3 is [ 0.00000E+00]: 0\n";
print OUTFID " The derivative wrt directions 1 & 3 is [ 0.00000E+00]: 0\n";
print OUTFID " The derivative wrt directions 2 & 3 is [ 0.00000E+00]: 0\n";
print OUTFID " The derivative wrt directions 1, 2 & 3 is [ 0.00000E+00]: 0\n";
print OUTFID " The number of versions for nj=2 is [1]: 1\n";
print OUTFID " The Xj(2) coordinate is [ 0.00000E+00]:    0\n";
print OUTFID " The derivative wrt direction 1 is [ 0.00000E+00]:   0\n";
print OUTFID " The derivative wrt direction 2 is [ 0.00000E+00]:  0\n";
print OUTFID " The derivative wrt directions 1 & 2 is [ 0.00000E+00]:   0\n";
print OUTFID " The derivative wrt direction 3 is [ 0.00000E+00]: 0\n";
print OUTFID " The derivative wrt directions 1 & 3 is [ 0.00000E+00]: 0\n";
print OUTFID " The derivative wrt directions 2 & 3 is [ 0.00000E+00]: 0\n";
print OUTFID " The derivative wrt directions 1, 2 & 3 is [ 0.00000E+00]: 0\n";
print OUTFID " The number of versions for nj=3 is [1]: 1\n";
print OUTFID " The Xj(3) coordinate is [ 0.00000E+00]:    0\n";
print OUTFID " The derivative wrt direction 1 is [ 0.00000E+00]:   0\n";
print OUTFID " The derivative wrt direction 2 is [ 0.00000E+00]:  0\n";
print OUTFID " The derivative wrt directions 1 & 2 is [ 0.00000E+00]:   0\n";
print OUTFID " The derivative wrt direction 3 is [ 0.00000E+00]: 0\n";
print OUTFID " The derivative wrt directions 1 & 3 is [ 0.00000E+00]: 0\n";
print OUTFID " The derivative wrt directions 2 & 3 is [ 0.00000E+00]: 0\n";
print OUTFID " The derivative wrt directions 1, 2 & 3 is [ 0.00000E+00]: 0\n";
print OUTFID "\n";


CORE::close(OUTFID);


sub DLambdaDXi3
{
  my ($d, $lambda, $mu, $theta) = @_;

  my $dx;
  my $dy;
  my $dz;
  my $ds;
  
  $dx = $d*sinh($lambda)*cos($mu*pi/180);
  $dy = $d*cosh($lambda)*sin($mu*pi/180)*cos($theta*pi/180);
  $dz = $d*cosh($lambda)*sin($mu*pi/180)*sin($theta*pi/180);  
  $ds = sqrt(($dx*$dx) + ($dy*$dy) + ($dz*$dz));  
  return (1/$ds);  
}


sub readNodes
{
  # Reads in nodes, but no multiples for nodes with more than 1 version
  my ($file) = @_;
  
  my $array = zeroes(4, 10000);
  my $index=0;  
  CORE::open(INFID, $file) || die "Error: Couldn't open $file\n";
  while (my $line = <INFID>)
  {
    if($line =~ m/ Node number.+:(.+)/)
    {
      $index++;
      PDL::Core::set $array,0,$index,$1;
    }
    if($line =~ m/ The Xj\((.+)\) coordinate is \[.+\]:(.+)/)
    {
      PDL::Core::set $array,$1,$index,$2;
    }
  }
  CORE::close(INFID);
  return($array);
}


