#Example 5f1

# You may want to use the optimized multiprocessor version of cmiss
# for this example.
#set num_threads 3

$INFLATION = 0;
$IVC = 1;

# Regions
$WALL = 1;
$CAVITY = 2;
$iterations = 15;

fem def param;r;full_big;example
fem define coor;r;rcCoord;example reg $WALL
fem define base;r;basisfuncs;example

fem define nodes;r;rc3;example reg $WALL
fem define elements;r;rc;example reg $WALL


fem define fibre;r;simple;example reg $WALL
fem define element;r;simple;example fibre reg $WALL

$name = "heartOriginal"
$file = $name
fem export nodes;$file as $name offset 0000
fem export elements;$file as $name offset_elem 0000

# Material groups
fem group nodes 1,18 as apexNodes
fem group nodes 2..5,19..22 as subApexNodes
fem group nodes 6..17,23..34 as normalNodes
fem group nodes 14..17,31..34 as baseNodes
# Boundary Condition groups
fem group nodes 1,18 as fixedApexNodes
fem group nodes 14..17,31..34 as fixedBaseNodes
fem group nodes 14,16 as y_axisNodes
fem group nodes 15,17 as z_axisNodes
 # Residual strains groups
fem group nodes 6..13 as nodes095
fem group nodes 1..5,14..22,31..34 as nodes100
fem group nodes 23..30 as nodes105

fem define equation;r;equation;example reg $WALL lock
fem define material;r;inhomo_nodes;example reg $WALL
fem define acti;r;calcium000;example reg $WALL

if($INFLATION)
{
  fem define initial;r;x_Base_node_restricted;example reg $WALL
  fem define solve;r;solve;example reg $WALL
  fem solve increment 0.0 iter 10 # due to residual strains
  fem solve increment 1 iter 20
  fem define initial;w;wall_inflated reg $WALL
} else #All ready solved the inflation phase
{
  fem define initial;r;wall_inflated;example reg $WALL
  fem define solve;r;solve;example reg $WALL
  fem evaluate residuals wrt geom_params
}

$name = "heartInflated"
$file = $name
fem export nodes;$file field as $name offset 1000
fem export elements;$file field as $name offset_elem 1000


if($IVC)
{
  fem define region;r;regions;example
  fem define coord;r;cavityCoord;example region $CAVITY
  fem define node;r;cavity;example region $CAVITY
  fem define element;r;cavity;example region $CAVITY

  # Set up equations, material properties and initial conditions
  # for cavity region
  fem define equation;r;equationRegions;example region $WALL,$CAVITY lock
  fem define material;r;materialCavityIVC;example region $CAVITY
  fem define initial;r;initCavity;example region $CAVITY
  #
  # Define cavity-to-wall coupling and solution procedure information
  fem define coupling;r;coupled;example
  fem define solve;r;solveRegion;example coupled region $WALL,$CAVITY
  #
  # Transfer wall deformation from the inflation problem to 
  # geometric dependent variables for the cavity region
  fem update solution coupled source_region $WALL
  fem update solution cavity_reference average 104 in 1 node 14..17 region $CAVITY

  fem evaluate residuals wrt geom_params coupled
  fem list element total region $CAVITY
  fem list element deformed total region $CAVITY
  #fem evaluate reactions coupled

  fem define active;r;calcium010;example
  fem solve coupled increment 0.0 iter $iterations error 1.0e-3
  fem define active;r;calcium014;example
  fem solve coupled increment 0.0 iter $iterations error 1.0e-3
  fem define active;r;calcium018;example
  fem solve coupled increment 0.0 iter $iterations error 1.0e-3

  $name = "heartIVC"
  $file = $name
  fem export nodes;$file field as $name offset 2000
  fem export elements;$file field as $name offset_elem 2000

  fem define initial;w;wall_endIVC region $WALL
  fem define initial;w;coupled_endIVC region $WALL,$CAVITY
  
} else
{
  fem define active;r;calcium018;example
  fem define initial;r;wall_endIVC;example region $WALL
  fem define solve;r;solve;example reg $WALL
  fem eval resid wrt geom reg $WALL
  #fem solve increment 0.0 iter 20 error 1.0e-1
} # if $IVC


