# If the example path is not set, default to current directory
if (!defined $example) {
    $example = "./";
}
# Drop off the trailing / in the example path
$chopped = chop $example;
if ($chopped ne "/") {
    $example .= $chopped;
}
#**************************************************************
#
# allocate memory for program arrays etc.
fem def para;r;memory;example
#**************************************************************
#
# Set up beam geometry
fem def coor 3,1
fem def base;r;TriCubicHerm;example #basis function for interpolation of 3d coords
fem def;add base;r;BiCubicHerm;example #for interpolation of coords on 2D surfaces
fem def;add base;r;TriLinearPress;example #interpolates hydrostatic pressure
fem def;add base;r;BiLinear;example 
##################################################################################################################################################
#NOTE
#
#The code to identify the undeformed from the deformed state currently works well when using geometry described with unit scale factors for the following reason
#When using non-unit scale factors, the forward problem calculates scale factors the first time when defining the undeformed state. These scale factors
#remain constant during the solution procedure. Therefore, if we use a deformed geometry with non-unit scale factors, the reverse problem does solve correctly
#but the nodal values determined during the procedure deviate slightly from the actual undeformed state values.
##################################################################################################################################################
# We load up the deformed state into memory. The deformed geometry is the initial guess at the undeformed geometry
fem def node;r;beam_def;example
fem def elem;r;beam_def;example
fem def fibre;r;beam_def;example
fem def elem;r;beam_def;example fibre
#
#group nodes that are against the wall for cantilever setup. Certain degrees of freedom for these nodes will be fixed in the ipinit file
#to simulate the cantilever setup
fem group node 1,6,11,16,21,26,31,36,41 as BACKNODES
fem def equa;r;mechanics;example #use 3D finite elasticity to simulate the deformation
#
#apart from the neo_hookean parameter, you need to give the density of the body..
#units for this model
#
# Density in gram per millimeter cubed 
# gravity m/(sec squared). 
# This means force is in mN. Therefore stress will be in kPa (kN/squared m)or (mN/squared mm)
# so neo-Hookean param is in kPa
fem def mate;r;neo_hookean;example 
#
#The gravity vector is set in the ipinit file. The initial conditions are those applied to the undeformed state.
#Once the initial conditions are defined, the first guess in the forward problem to the deformed state is the undeformed state.
#As the deformed is loaded into the undeformed state memory, the deformed state is correctly copied into the deformed geometry memory. 
fem def init;r;gravity;example
#
#Define the nonlinear equations solution algorithm to be used
#I have chosen modified newton with LU factorisation. The LU factorisation
# can be slow, but with modified newton, the total time taken can actually be reduced
fem def solv;r;modif_newton;example
#**************************************************************
#
# solve the problem for reference state
#During the solution procedure, the undeformed geometry parameters in memory are perturbed rather than the deformed geometry parameters. The stiffness
#matrix contains derivatives with respect to the undeformed geometry coordinates and the hydrostatic pressure variables for the deformed state. 

fem solv undeformed iter 100
#**************************************************************
#
#view the undeformed state on the gx window
fem def wind on xz;c
fem draw line #shows predicted undeformed state
fem draw line def dot 
fem export node;beam_inv as beam_inv offset 100
fem export elem;beam_inv as beam_inv offset_elem 100 offset_node 100
fem def node;w;beam_inv #writing out the predicted undeformed state
