# If the example path is not set, default to current directory
if (! defined $example) {
    $example = "./";
}
# Drop off the trailing / in the example path
$chopped = chop $example;
if ($chopped ne "/") {
    $example .= $chopped;
}

$filename = "scamba";
$RADIUS = 1;
$INSPIRATION = 1;
$EXPIRATION = 2;
$dt = 0.05;
$T_insp = 1.0;
$T_expn = 1.0;
$NUM_EXPORTS=1;
$time = 0.0;
$flow_in = 0.5; #litres
$flow_out = -0.5; #litres
$v_t = $flow_in * $T_insp; #in litres
$FIXED_NODE = 1;
$FLUX_NODE = 1;
$initial = 0.0;
$concentration = 1.0;
$material = "o2n2";
$NUM_BREATHS = 10;
$N=0;

fem de param;r;"$example/$filename";      #set up parameters for array dimensioning
fem de coor;r;"$example/$filename";        #3d rectangular cartesian
fem de base;r;"$example/$filename";        #linear basis function with 4 gauss points
fem de mesh;r;"$example/$filename" as conducting field; 
fem de;add mesh;c airway multi_hbw basis_function 1 as respiratory field radius_field 1 alveolar_field 2; 
fem update mesh geometry elem conducting radius_field 1 anatomical weibel;
fem update mesh geometry elem respiratory radius_field 1 anatomical hbw;
fem group node in element respiratory as respiratory;

#DEFINE THE PROBLEM SETUP FOR INSPIRATION
fem de equa;r;"$example/$filename" class $INSPIRATION;
fem de mate;r;"$example/$material" class $INSPIRATION;
fem de moti;c lung elem respiratory uniform flow_field 3 inlet_flow $flow_in class $INSPIRATION;
fem de init;c lung fixed node $FIXED_NODE conc $concentration initial $initial class $INSPIRATION;
fem de solve;r;"$example/${filename}_insp" name $material;

#DEFINE THE PROBLEM SETUP FOR EXPIRATION
fem de equa;r;"$example/$filename" class $EXPIRATION;
fem de mate;r;"$example/$material" class $EXPIRATION;
fem de moti;c lung elem respiratory uniform flow_field 3 inlet_flow $flow_out class $EXPIRATION;
fem de init;c lung flux node $FLUX_NODE class $EXPIRATION;
fem de solve;r;"$example/${filename}_expn" class $EXPIRATION name $material;

$restart = "";
$time_end = 0.0;

for (my $j = 0; $j < $NUM_BREATHS; $j++){

    for (my $i = 0; $i < $NUM_EXPORTS; $i++){
	$time_end = $time_end + $T_insp/$NUM_EXPORTS;
	print "FEM solve $restart to $time_end delta_t $dt class $INSPIRATION\n";
	fem solve $restart to $time_end delta_t $dt class $INSPIRATION;
	$N++;
	$restart = "restart";

    }
    fem update solution class $EXPIRATION substitute solution class $INSPIRATION;
    
    for (my $i = 0; $i < $NUM_EXPORTS; $i++){
	$time_end = $time_end + $T_expn/$NUM_EXPORTS;
	print "FEM solve $restart to $time_end delta_t $dt class $EXPIRATION\n";
	fem solve $restart to $time_end delta_t $dt class $EXPIRATION;
	$N++;
    }
    
    fem update solution class $INSPIRATION substitute solution class $EXPIRATION;
    fem update solution class $INSPIRATION nodes $FIXED_NODE substitute constant $concentration;
}

#fem evaluate pulmonary Sn for $NUM_BREATHS ev_start 0.75 ev_end 0.95 vt $v_t;
