#Example_92  Gas mixing in a symmetric conducting/LPM respiratory model
#This exmaple simulates mixing of sulpherhexafloride with nitrogen
#in a full lung model.  The model uses a symmetric conducting airway
#system, based on Weibel's model A down to generation 16.  Because the 
#model is symmetric, only a single pathway is considered.  Attached to
#the end of the terminal conducting airway is a lumped parameter model 
#of a single pulmonary acinus, calibrated using simulations in the 
#sca-mba model described in example 91.

# If the example path is not set, default to current directory
if (! defined $example) {
    $example = "./";
}
# Drop off the trailing / in the example path
$chopped = chop $example;
if ($chopped ne "/") {
    $example .= $chopped;
}

$filename = "scalpm";
$RADIUS = 1;
$ALVEOLI=2;
$INSPIRATION = 1;
$EXPIRATION = 2;
$dt = 0.05;
$T_insp = 1.0;
$T_expn = 1.0;
$time = 0.0;
$flow_in = 0.5; #litres
$flow_out = -0.5; #litres
$v_t = $flow_in * $T_insp; #in litres
$FIXED_NODE = 1;
$FLUX_NODE = 1;
$initial = 0.0;
$concentration = 1.0;
$NUM_BREATHS = 10;
$NUM_EXPORTS = 1;

fem de param;r;"$example/$filename";
fem de coor;r;"$example/$filename";
fem de base;r;"$example/$filename";

#CREATE SYMMETRIC CONDUCTING MODEL AND LUMPED PARAMETER RESPIRATORY MODEL
fem de mesh;r;"$example/$filename" as conducting field;
fem de;add mesh;c airway lumped_parameter basis_function 1 as respiratory field radius_field $RADIUS alveolar_field $ALVEOLI;
fem group node in element respiratory as respiratory;
fem evaluate ordering;

#CALCULATE RADII AND ALVEOLAR CROSS-SECTIONAL AREA
fem update mesh geometry elem conducting radius_field $RADIUS anatomical weibel;
fem update mesh geometry elem respiratory radius_field $RADIUS lumped volume 51.8 length 1 aa_ratio 0.5;

#DEFINE THE PROBLEM SETUP FOR INSPIRATION
fem de equa;r;"$example/$filename" class $INSPIRATION;
fem de mate;r;"$example/$filename" class $INSPIRATION;
fem de moti;c lung elem respiratory uniform flow_field 3 inlet_flow $flow_in class $INSPIRATION;
fem de init;c lung fixed node $FIXED_NODE conc $concentration initial $initial class $INSPIRATION;
fem de solve;r;"$example/$filename";

#DEFINE THE PROBLEM SETUP FOR EXPIRATION
fem de equa;r;"$example/$filename" class $EXPIRATION;
fem de mate;r;"$example/$filename" class $EXPIRATION;
fem de moti;c lung elem respiratory uniform flow_field 3 inlet_flow $flow_out class $EXPIRATION;
fem de init;c lung flux node $FLUX_NODE class $EXPIRATION;
fem de solve;r;"$example/$filename" class $EXPIRATION;

#SOLVE FOR FIRST_BREATH
$restart = "";
$time_end = 0.0;

for (my $j = 0; $j < $NUM_BREATHS; $j++){

    for (my $i = 0; $i < $NUM_EXPORTS; $i++){
	$time_end = $time_end + $T_insp/$NUM_EXPORTS;
	print "FEM solve $restart to $time_end delta_t $dt class $INSPIRATION\n";
	fem solve $restart to $time_end delta_t $dt class $INSPIRATION;
	$N++;
	$restart = "restart";

    }

    fem update solution class $EXPIRATION substitute solution class $INSPIRATION;
    
    for (my $i = 0; $i < $NUM_EXPORTS; $i++){
	$time_end = $time_end + $T_expn/$NUM_EXPORTS;
	print "FEM solve $restart to $time_end delta_t $dt class $EXPIRATION\n";
	fem solve $restart to $time_end delta_t $dt class $EXPIRATION;
	$N++;
    }
    
    fem update solution class $INSPIRATION substitute solution class $EXPIRATION;
    fem update solution class $INSPIRATION nodes $FIXED_NODE substitute constant $concentration;
}

