# Example b132 - HMT
#
# Solves the HMT model for a single cell, with a step change in
# length defined by a time varying boundary condtion set by using
# a time variable. The value of [Ca]i is also set via a time variable.

sub hmtCaiTransient {
    # Routine to define a time variable describing the [Ca]i transient
    # from the HMT paper
    $Cai_init = 0.0001;  # mM
    $Cai_max = 0.001; # mM
		$Cai_Tinit = 50.0;# ms
    $Tau = 60;        # ms
    $numPoints=1000;
    $variableName = "cai";

	CORE::open(IPTIME,">tmp-cai-HMT.iptime") || die "Cannot open iptime file: $!\n";
    print IPTIME " CMISS Version 1.21 iptime File Version 3\n";
    print IPTIME " Heading:\n";
    print IPTIME "\n";
    print IPTIME " Enter the name of the time variable [EXIT]: $variableName\n";
    print IPTIME "\n";
    print IPTIME " Do you want this time variable to be periodic [N]? N\n";
    print IPTIME "\n";
    print IPTIME " Enter the number of time points to be set [2]: $numPoints\n";
    print IPTIME " Enter the time variable value before time point 1 [0.0]: $Cai_init\n";
    for ($point=1;$point<=$numPoints;$point++){
        $time=1000/$numPoints*($point-1);
				if ($time < $Cai_Tinit) {
						$Cai = $Cai_init;
				} else {
						$t = $time - $Cai_Tinit;
						$Cai = $Cai_init+($Cai_max-$Cai_init)*($t/$Tau)*exp(1-$t/$Tau);
				}
        print IPTIME " Enter the time for time point $point [0.0]: $time\n";
        print IPTIME " Enter the time variable value at time point $point [0.0]: $Cai\n";
    }
    print IPTIME " Enter the time variable value after time point $numPoints [0.0]: $Cai_init\n";
    print IPTIME "\n";
    print IPTIME " The type of interpolation is [2]:\n";
    print IPTIME "   (1) Constant\n";
    print IPTIME "   (2) Linear Lagrange\n";
    print IPTIME "  *(3) Quadratic Lagrange\n";
    print IPTIME "  *(4) Cubic Lagrange\n";
    print IPTIME "  *(5) Cubic Hermite\n";
    print IPTIME "    2\n";
    print IPTIME " Enter the name of the time variable [EXIT]: EXIT\n";
	CORE::close IPTIME;
    fem define time;r;tmp-cai-HMT;
    unlink "tmp-cai-HMT.iptime";
} # sub hmtCaiTransient

# If the example path is not set, default to current directory
if (! defined $example) {
    $example = "./";
}
# Drop off the trailing / in the example path
$chopped = chop $example;
if ($chopped ne "/") {
    $example .= $chopped;
}

fem define parameters;r;$example/hmt

fem define node;d
fem define base;d
fem define element;d
fem define grid;d
fem update grid geometry
fem update grid metric

fem define equation;r;$example/hmt

fem define cell;r;$example/hmt
fem define material;r;$example/hmt cell
# Need to define the material conductivities even though they are not 
# really used
fem define material;r;$example/hmt
fem update grid material

# read in the time variable describing the step length change
fem define time;r;$example/step
# and the concentration of intracellular Ca++
&hmtCaiTransient();
# assign the time variable to use for the time varying boundary 
# condition
fem define initial;r;$example/hmt

fem define solve;r;$example/hmt

fem solve to 0

fem open history;hmt write variables yqs niqslist 1,2,3,4,5,6,7 binary

for $time ( 0..999 ) 
{
#  Solve for the current time
fem solve restart to $time

#  Write the membrane potential to the history file
fem write history time $time variables yqs binary
}

# Close the history file
fem close history binary


# Write out signal files for all the state variables.
fem evaluate electrode;hmt_all history hmt from grid yqs iy 1..7 binary
fem define export;r;$example/hmt_all
fem export signal;hmt_all electrode signal hmt_all names cell

