# Example b17:
# 2D sheet with N98-HMT used for active tension calculation - testing
# reading and writing of binary signal files.
#

# If the example path is not set, default to current directory
if (! defined $example) {
    $example = "./";
}
# Drop off the trailing / in the example path
$chopped = chop $example;
if ($chopped ne "/") {
    $example .= $chopped;
}

set echo;

$FORMAT = "binary";

# Set the output directory
$output = ".";
if (! -d $output) {
    mkdir $output,0700;
}

# The two class definitions
$MECHANICS = 1;
$CELL = 2;

# Set up the geometry 4x4 elements, bicubic Hermite basis
fem define parameter;r;$example/emech;
##**fem define node;r;$example/emech-8x8-cubic;
fem define node;r;$example/emech-4x4-cubic;
# Need a bi-cubic Hermite for geometry and fibres and a bi-linear
# for grid-based FEM
fem define base;r;$example/emech-cubic-linear;
fem define element;r;$example/emech-4x4-cubic;
# Fibres varying from 10 deg to 90 deg and back to 10 deg - with
# cubic Hermite interpolation
fem define fibre;r;$example/emech-4x4-109010;
fem define element;r;$example/emech-4x4-cubic fibre;

# Need to calculate the undeformed nodal derivatives
fem update node derivative 1 linear;
fem update node derivative 2 linear;
fem update node derivative 3 linear;

# Node/element groups to set BCs
# - the corners group will be fixed in space
# - the fixed_y group will be fixed in the y-direction
fem group node external as bdy_nodes;
#fem group node 1,10,19,28,37,46,55,64,73 as corners
fem group node 1 as corners;
fem group node 5 as fixed_y;
fem group element 1..16 as ALL_ELEMENTS;

# Set up the finite elasticity
fem define equation;r;$example/mech-cubic class $MECHANICS lock;
fem define material;r;$example/mech-cubic class $MECHANICS;
# use the cellular HMT active tension
fem define active;r;$example/mech-cubic class $MECHANICS;
fem define initial;r;$example/mech-cubic class $MECHANICS;
# Define the Newton iterative solver
fem define solve;r;$example/newton class $MECHANICS;

# Set up the N98-HMT cellular modelling - used to calculate the active 
# tension and wavefront propagation
fem define grid;r;$example/cell-18x18 class $CELL;
fem update grid geometry;
fem update grid metric;
#fem group grid xi1=0 oneoff as stimulus1_site;
fem group grid grid 1,2,3,10,11,12,19,20,21 as stimulus1_site;
fem group grid xi1 low element 3,11 as stimulus2_site;
fem group grid element ALL_ELEMENTS as ALL_GRID_POINTS;
# Use N98-HMT with grid-based FEM
fem define equation;r;$example/cell-n98_hmt class $CELL;
fem define material;r;$example/cell-n98_hmt class $CELL;
fem define cell;r;$example/cell-n98_hmt class $CELL;
fem define material;r;$example/cell-n98_hmt cell class $CELL;
fem update grid material class $CELL;
fem define initial;r;$example/cell-n98_hmt class $CELL;
# use the Adams-Moulton integrator
fem define solve;r;$example/cell-hmt-implicit-adams class $CELL;

# Initialise the cell integration
fem solve class $CELL to 0;

# Tension index for grid (YQS) and Gauss (YG) variables
##fem inquire cell_variable Tension return_variables T_GRIDARRAY,T_GRIDIDX;
fem inquire cell_variable IsometricTension return_variables T_GRIDARRAY,T_GRIDIDX;
# Fibre extension ratio index for grid (YQS) and mechanics variables
fem inquire cell_variable ExtensionRatio return_variables L_GRIDARRAY,L_GRIDIDX;
$L_MECHIDX = 1;
# and previous fibre extension ratio index for grid (YQS) and mechanics variables
fem inquire cell_variable ExtensionRatioPrevious return_variables L_GRIDARRAY_PREV,L_GRIDIDX_PREV;
# [Ca]i index for grid (YQS)
fem inquire cell_variable Cai return_variables CAI_GRIDARRAY,CAI_GRIDIDX;
# Vm index for grid (YQS)
fem inquire cell_variable Vm return_variables VM_GRIDARRAY,VM_GRIDIDX;


# Compute the CMISS signal files from the history file
set echo;
fem evaluate electrode;$output/Vm history $example/cell-n98_hmt from grid yqs iy $VM_GRIDIDX $FORMAT class $CELL;
fem evaluate electrode;$output/cai history $example/cell-n98_hmt from grid yqs iy $CAI_GRIDIDX $FORMAT class $CELL;
fem evaluate electrode;$output/tension history $example/cell-n98_hmt from grid yqs iy $T_GRIDIDX $FORMAT class $CELL;
fem evaluate electrode;$output/extension_ratio history $example/cell-n98_hmt from grid yqs iy $L_GRIDIDX $FORMAT class $CELL;

# Export to UnEMAP signal files
fem define export;r;$example/cell-4x4;
fem export signal;$output/Vm electrode signal $output/Vm;
fem export signal;$output/cai electrode signal $output/cai;
fem export signal;$output/tension electrode signal $output/tension;
fem export signal;$output/extension_ratio electrode signal $output/extension_ratio;

quit;


