
#Example b2118 - Explicit 2d Luo-Rudy over a 2d strip of elements

# Assign some useful variables
# The file format

$FORMAT = "binary"

# The signals to export ???

$VARIABLES = 1

# The finish time of the integration

$TEND = 10

# The grid points to be stimulated

$STIMULUS_SITE = "xi1=low oneoff element 1"

# The name of the output files

$OUTPUT_FILE = "lr"

# Define the model size
fem define parameter;r;biline;example

# Define the model geometry and grid scheme
fem define node;r;biline;example
fem define base;r;biline;example
fem define elem;r;biline;example
fem define grid;r;biline;example
fem update grid geometry
fem update grid metric

# Define some grid groups
fem group grid external as boundary
#fem group grid grid 2,11,20,29,38,47,56,65,74 as stimulus_site
{
  my $command = "fem group grid $STIMULUS_SITE as stimulus_site";
  cmiss $command or die "Error in cmiss command \"$command\".\n";
}

# Define the model - Monodomain, Luo-Rudy
fem define equa;r;biline;example

# Define the cellular material parameters
fem define cell;r;lr_double_pulse;example

# Define the spatially varying cellular parameters
fem def mate;r;biline;example cell

# Define the continuum material parameters
fem define mate;r;biline;example
fem update grid material

# Define the initial conditions - zero flux on the boundary
fem define init;r;biline;example

# Use the adaptive Adams solver, fully explicit
fem define solv;r;biline;example

# Initialise the solve
fem solve to 0

# Create a history file to store the required variables at all 
# grid points
fem open history;$OUTPUT_FILE write variables yq niqlist $VARIABLES $FORMAT

# Export the geometry
fem export node;$OUTPUT_FILE as biline
fem export element;$OUTPUT_FILE as biline
fem export element;numbers as biline grid_numbers

# Export the cellular material parameters
fem export element;parameters as biline field cell

# Integrate the model

for $TIME ( 0..$TEND ) 
{
  # Display the current time
  print " TIME = ".sprintf("%7.1f",${TIME})." ms\n"

  # Solve the model for the current time
  fem solve restart to $TIME

  # Write the new variable values to the history file
  fem write history time $TIME variables yq $FORMAT

  # Export the solution field
  fem export element;${OUTPUT_FILE}."_1".sprintf("%05d",$TIME) field as biline
}

# Close the history file
fem close history $FORMAT

# Export the UnEMAP signal file
fem evaluate electrode;$OUTPUT_FILE history $OUTPUT_FILE from grid yq iy $VARIABLES $FORMAT
fem define export;r;biline;example
fem export signal;$OUTPUT_FILE electrode signal $OUTPUT_FILE
