# Example b211k - Explicitly solve the LR2 model over a square sheet 
# of tissue, using the Adams integrator.

# If the example path is not set, default to current directory
if (! defined $example) {
    $example = "./";
}
# Drop off the trailing / in the example path
$chopped = chop $example;
if ($chopped ne "/") {
    $example .= $chopped;
}

#Set up problem size
fem define para;r;$example/test;

#Define geometry of the problem
#  Define underlying geometry
fem define base;r;$example/test;
fem define mesh;r;$example/test;
#  Define grid geometry
fem define grid;r;$example/test;
fem update grid geometry;
fem update grid metric;
#  Some useful groups
fem group grid external as boundary;
fem group grid grid 1889,2353,1881,1888,1946 as stimulus_site;

#Define equation to solve
#  Define monodomain with LR2 kinetics
fem define equa;r;$example/test;

#Define material properties for the problem

#  Define cellular material properties
fem define cell;r;$example/lr;
#  Define cell variants and spatially varying material parameters
if ($TESTING) {
    # When testing we don't won't to apply a stimulus as we start from a
    # solution that is already stimuluated
    fem define mate;r;$example/test_noStim cell;
} else {
    fem define mate;r;$example/test cell;
}
#  Define the continuum material parameters
fem define mate;r;$example/test;
fem update grid material;

#Define continuum initial conditions
#  Set no-flux boundary condition
fem define init;r;$example/test;

#Define the explicit solve, using the adams integrator
fem def solv;r;$example/test;


# The geometry could be exported to CMGUI via these commands
fem export node;test as sheet;
fem export elem;test as sheet;

#Solve the problem

#  Initialise the solve
fem solve to 0;

#  Open a binary history file to store the membrane potential (YQ(1))
fem open history;test write variables yq niqlist 1 binary unit 21;

# For testing, we simply read in a previous solution and solve one time 
# step.
if ($TESTING) {
    set echo;
    fem read matrix;$example/test binary;
    fem solve restart to 1;
    fem write history time 10 variables yq binary unit 21;
    fem export elem;test_1010 field as sheet;
} else {
    # Loop through time
    for $HUN ( 0..0 ) {
        for $TEN ( 0..1 ) {
            for $UNIT ( 0..9 ) {
                # Solve for the current time
                fem solve restart to "${HUN}${TEN}$UNIT";
                # Write the membrane potential to the history file
                fem write history time "${HUN}${TEN}$UNIT" variables yq binary unit 21;
                #  Export the field to CMGUI
                fem export elem;test_1${HUN}${TEN}$UNIT field as sheet;
            }
        }
    }
}

#  Close the history file
fem close history binary unit 21;

if (!$TESTING) {
    # Calculate the binary signal file from the history file
    fem evaluate electrode;test history test from grid binary;
    
    # Define UnEmap export properties
    fem define export;r;$example/test;

    # Export the binary signal file to UnEmap
    fem export signal;test electrode signal test;
}

