#Example b2141
#
# This example demonstrates how to couple a single ventricle
#  and a single torso region through a multiple surface fixed
#  point iteration
#

# Problem classes
$CAVITY=1 
$VENTCL=2 
$BEMHRT=3 
$GRIDHT=4 

# Problem regions
$TLPL=1 
$BLPL=2
$TMEM=3 
$EXTC=4 

# Iteration variables
$NUM_ITERS=0 
$MAX_ITERS=50 
$FINISHED=0
$POTE_TOL=0.1 
$FLUX_TOL=0.1 
$POTE1=1.0
$FLUX1=1.0
$POTE2=1.0
$FLUX2=1.0
$ALPHA1=0.0 
$ALPHA2=0.7 

# Define problem geometry
fem define para;r;torsoslice;example
fem define regi;r;torsoslice;example
fem define node;r;cavity3;example reg $CAVITY
fem define node;r;ventcl3;example reg $VENTCL
fem define node;r;bemhrt3;example reg $BEMHRT
fem define node;r;gridht;example  reg $GRIDHT
fem define base;r;linear;example
fem define elem;r;cavity3;example reg $CAVITY
fem define elem;r;ventcl3;example reg $VENTCL
fem define elem;r;bemhrt3;example reg $BEMHRT
fem define elem;r;gridht;example  reg $GRIDHT

# Define finite difference grid
fem define grid;r;gridht;example reg $GRIDHT
fem update grid geom reg $GRIDHT
fem update grid metric reg $GRIDHT
fem group grid xi2=0 as endocardium reg $GRIDHT
fem group grid xi2=1 as epicardium reg $GRIDHT
fem group grid external as boundary reg $GRIDHT
fem group grid line 1 xidirn 2 as stimulus reg $GRIDHT

# Create node to grid mapping
fem update node grid grid_reg $GRIDHT bem_reg $BEMHRT
fem update node geomet from grid reg $BEMHRT

# Define equations
fem define equa;r;cavity;example reg $CAVITY class $TLPL lock
fem define equa;r;ventcl;example reg $VENTCL class $BLPL lock
fem define equa;r;gridht;example reg $GRIDHT class $TMEM,$EXTC lock

# Define material parameters
fem define mate;r;cavity;example reg $CAVITY class $TLPL
fem define mate;r;ventcl;example reg $VENTCL class $BLPL
fem define mate;r;gridht;example reg $GRIDHT class $TMEM,$EXTC
fem define cell;r;gridht;example reg $GRIDHT class $TMEM,$EXTC
fem update grid material reg $GRIDHT class $TMEM,$EXTC

# Define initial conditions
fem define init;r;cavity;example reg $CAVITY class $TLPL
fem define init;r;ventcl;example reg $VENTCL class $BLPL
fem define init;r;gridht;example reg $GRIDHT class $TMEM,$EXTC

# Define grid to boundary element coupling
fem up elem region $GRIDHT,$VENTCL,$CAVITY class $EXTC,$BLPL,$TLPL grids boundary

# Define solution parameters
fem define solv;r;gridht;example reg $GRIDHT class $TMEM
fem define solv;r;gridht;example reg $GRIDHT class $EXTC
fem define solv;r;cavity;example reg $CAVITY class $TLPL
fem define solv;r;ventcl;example reg $VENTCL class $BLPL

# Solve the passive torso problems
fem solve reg $CAVITY class $TLPL
fem solve reg $VENTCL class $BLPL

# Solve the bidomain problem
fem solve activ coupled to 0 reg $GRIDHT,$CAVITY,$VENTCL class $TMEM,$EXTC,$TLPL,$BLPL

for $TEN ( 0..0 ) {
  for $UNIT (0..0) {
    for $DECI ( 0..9 ) {
      for $HUND ( 0..9 ) {

        # Solve a bidomain time step
        fem solve activ coupled transmembrane restart to "$TEN$UNIT".'.'."$DECI$HUND" reg $GRIDHT class $TMEM,$EXTC noupdate
        $NUM_ITERS = 0
        $FINISHED = 0

        # Find a converged solution
        while( !$FINISHED && $NUM_ITERS<=$MAX_ITERS ) {
          $NUM_ITERS++ 

          fem solve activ coupled extra rest reg $GRIDHT,$CAVITY,$VENTCL cl $TMEM,$EXTC,$TLPL,$BLPL noupdate static alpha $ALPHA1
          fem update init from grid region $CAVITY,$BEMHRT,$GRIDHT class $EXTC,$TLPL,$EXTC alpha $ALPHA2
          fem update init from grid region $VENTCL,$BEMHRT,$GRIDHT class $EXTC,$BLPL,$EXTC alpha $ALPHA2
          fem solve reg $CAVITY class $TLPL
          fem solve reg $VENTCL class $BLPL
          fem check conv grid_reg $BEMHRT grid_cl $EXTC bem_reg $CAVITY bem_cl $TLPL grid_update_cl $EXTC return POTE1 FLUX1
          fem check conv grid_reg $BEMHRT grid_cl $EXTC bem_reg $VENTCL bem_cl $BLPL grid_update_cl $EXTC return POTE2 FLUX2

          $FINISHED = (abs($POTE1)<=$POTE_TOL && abs($FLUX1)<=$FLUX_TOL && abs($POTE2)<=$POTE_TOL && abs($FLUX2)<=$FLUX_TOL)
          print("Iteration $NUM_ITERS\n");
          }

        # Update the transmembrane potential field
        fem solve activ coupled trans rest reg $GRIDHT,$CAVITY,$VENTCL class $TMEM,$EXTC,$TLPL,$BLPL update static alpha $ALPHA1

        if($NUM_ITERS<$MAX_ITERS) {
          print("Solution converged in $NUM_ITERS iterations\n");
        }
        else {
          print("Solution did not converge in $MAX_ITERS iterations\n");
          print("Outside potential $POTE1 Outside flux $FLUX1\n");
        }
      }
    }
  }
}
