#Example b2143
#
# This example demonstrates how to couple a single ventricle
#  by direct assembly while iterating on the epicardial 
#  surface to get a continuous solution.
#

# Problem classes
$TLPL=1 
$BLPL=2 
$TMEM=3 
$EXTC=4 

# Problem regions
$CAVITY=1 
$BLOOD=2 
$BEMHT=3 
$MUSCLE=4 
$GRIDHT=5 

# Iteration variables
$NUM_ITERS=0 
$MAX_ITERS=50 
$FINISHED=0
$POTE_TOL=0.1 
$FLUX_TOL=0.1 
$POTE1=1.0
$FLUX1=1.0
$ALPHA1=0.0 
$ALPHA2=0.5 

# Define problem geometry
fem define para;r;fembem;example
fem define regi;r;fembem;example
fem define node;r;refx4;example reg all
fem define base;r;fembem;example
fem define elem;r;refx4;example reg all
fem define fibr;r;refx4;example reg $GRIDHT
fem define elem;r;test;example fibre reg $GRIDHT

# Define finite difference grid
fem define grid;r;heart;example reg $GRIDHT
fem update grid geometry reg $GRIDHT
fem update grid metric reg $GRIDHT
fem group grid xi2=0 as endocardium reg $GRIDHT
fem group grid xi2=1 as epicardium  reg $GRIDHT
fem group grid external as boundary reg $GRIDHT
fem group grid line 849 xidirn 2 as stimulus reg $GRIDHT

# Create node to grid mapping
fem update node grid grid_reg $GRIDHT bem_reg $BEMHT
fem update node geomet from grid reg $BLOOD

# Define equations
fem define equa;r;torso;example reg $CAVITY,$MUSCLE class $TLPL lock
fem define equa;r;blood;example reg $BLOOD class $BLPL lock
fem define equa;r;heart;example reg $GRIDHT class $TMEM,$EXTC

# Define material parameters
fem define mate;r;torso;example reg $CAVITY,$MUSCLE class $TLPL
fem define mate;r;blood;example reg $BLOOD class $BLPL
fem define mate;r;heart;example reg $GRIDHT class $TMEM,$EXTC
fem define cell;r;heart;example reg $GRIDHT class $TMEM,$EXTC
fem update grid material reg $GRIDHT class $TMEM,$EXTC

# Define initial conditions
fem define init;r;torso;example reg $CAVITY,$MUSCLE class $TLPL
fem define init;r;blood;example reg $BLOOD class $BLPL
fem define init;r;heart;example reg $GRIDHT class $TMEM,$EXTC

# Define torso region coupling
fem define coup;r;torso;example class $TLPL

# Define grid to boundary element coupling
fem up elem region $GRIDHT,$CAVITY class $EXTC,$TLPL grids epicardium

# Define solution parameters
fem define solv;r;heart;example reg $GRIDHT class $TMEM
fem define solv;r;heart2;example reg $GRIDHT class $EXTC
fem define solv;r;torso;example coupled reg $CAVITY,$MUSCLE class $TLPL
fem define solv;r;blood;example reg $BLOOD class $BLPL

# Solve the passive torso problems
fem solve coupled reg $CAVITY,$MUSCLE class $TLPL
fem solve reg $BLOOD class $BLPL 

# Solve the bidomain problem
fem solve activ coupled to 0 reg $GRIDHT,$BLOOD,$CAVITY class $TMEM,$EXTC,$BLPL,$TLPL grid endocardium cptype 3

for $TEN ( 0..0 ) {
  for $UNIT ( 0..0 ) {
    for $DECI ( 0..4 ) {
      for $HUND ( 0,2,4,6,8 ) {

        # Solve a bidomain time step
        fem solve activ coupled transmembrane restart to "$TEN$UNIT".'.'."$DECI$HUND" reg $GRIDHT class $TMEM,$EXTC noupdate
        $NUM_ITERS = 0
        $FINISHED = 0

        # Find a converged epicardial solution
        while( !$FINISHED && $NUM_ITERS<=$MAX_ITERS ) {
          $NUM_ITERS++ 

          fem solve activ coupled extra rest reg $GRIDHT,$BLOOD,$CAVITY cl $TMEM,$EXTC,$BLPL,$TLPL noupdate static alpha $ALPHA1 grid endocardium cptype 3
          fem update init from grid region $CAVITY,$BEMHT,$GRIDHT class $EXTC,$TLPL,$EXTC alpha $ALPHA2
          fem solve coupled reg $CAVITY,$MUSCLE class $TLPL
          fem check conv grid_reg $BEMHT grid_cl $EXTC bem_reg $CAVITY bem_cl $TLPL grid_update_cl $EXTC return POTE1 FLUX1

          $FINISHED = (abs($POTE1)<=$POTE_TOL && abs($FLUX1)<=$FLUX_TOL)
          print("Iteration $NUM_ITERS\n");
        }

        # Update the transmembrane potential field
        fem solve activ coupled trans restart reg $GRIDHT,$BLOOD,$CAVITY class $TMEM,$EXTC,$BLPL,$TLPL update static grid endocardium cptype 3

        if($NUM_ITERS<$MAX_ITERS) {
          print("Solution converged in $NUM_ITERS iterations\n");
        }
        else {
          print("Solution did not converge in $MAX_ITERS iterations\n");
          print("Outside potential $POTE1 Outside flux $FLUX1\n");
        }
      }
    }
  }
}
