#Example b2146
#
# This example demonstrates how to couple one ventricular
#  region and one torso cavity region by direct assembly.
#

# Problem classes
$CAVITY=1 
$VENTCL=2 
$BEMHRT=3 
$GRIDHT=4 

# Problem regions
$TLPL=1 
$BLPL=2 
$TMEM=3 
$EXTC=4 

# Define problem geometry
fem define para;r;torsoslice;example
fem define regi;r;torsoslice;example
fem define node;r;cavity3;example reg $CAVITY
fem define node;r;ventcl3;example reg $VENTCL
fem define node;r;bemhrt3;example reg $BEMHRT
fem define node;r;gridht;example  reg $GRIDHT
fem define base;r;linear;example
fem define elem;r;cavity3;example reg $CAVITY
fem define elem;r;ventcl3;example reg $VENTCL
fem define elem;r;bemhrt3;example reg $BEMHRT
fem define elem;r;gridht;example  reg $GRIDHT

# Define finite difference grid
fem define grid;r;gridht;example reg $GRIDHT
fem update grid geom reg $GRIDHT
fem update grid metric reg $GRIDHT
fem group grid external as boundary reg $GRIDHT
fem group grid line 1 xidirn 2 as stimulus reg $GRIDHT

# Create node to grid mapping
fem update node grid grid_reg $GRIDHT bem_reg $BEMHRT
fem update node geomet from grid reg $BEMHRT

# Define equations
fem define equa;r;cavity;example reg $CAVITY class $TLPL lock
fem define equa;r;ventcl;example reg $VENTCL class $BLPL lock
fem define equa;r;gridht;example reg $GRIDHT class $TMEM,$EXTC lock

# Define material parameters
fem define mate;r;cavity;example reg $CAVITY class $TLPL
fem define mate;r;ventcl;example reg $VENTCL class $BLPL
fem define mate;r;gridht;example reg $GRIDHT class $TMEM,$EXTC
fem define cell;r;gridht;example reg $GRIDHT class $TMEM,$EXTC
fem update grid material reg $GRIDHT class $TMEM,$EXTC

# Define initial conditions
fem define init;r;cavity;example reg $CAVITY class $TLPL
fem define init;r;ventcl;example reg $VENTCL class $BLPL
fem define init;r;gridht;example reg $GRIDHT class $TMEM,$EXTC

# Define solution parameters
fem define solv;r;gridht;example reg $GRIDHT class $TMEM
fem define solv;r;gridht;example reg $GRIDHT class $EXTC
fem define solv;r;cavity;example reg $CAVITY class $TLPL
fem define solv;r;ventcl;example reg $VENTCL class $BLPL

# Solve the passive torso problems
fem solve reg $CAVITY class $TLPL
fem solve reg $VENTCL class $BLPL

# Solve the bidomain problem for 10ms
fem solve activ coupled to 10 reg $GRIDHT,$VENTCL,$CAVITY class $TMEM,$EXTC,$BLPL,$TLPL grid boundary cptype 6 fixed_node 17
