$tend = 600;
$tstep = 1.0;

$outputDir = "output";
$imageDir = "output/images";

if (! defined $movie) {
		$movie = 0;
}
if (! defined $images) {
		$images = 0;
}

if ($images) {
		if (! -d $imageDir) {
				(mkdir $imageDir,0700) or die "Unable to create the image directory ($imageDir): $!";
		}
}

# Read in the undeformed geometry
gfx read node $outputDir."/sheet"
gfx read element $outputDir."/sheet"
# And the deformed geometry
gfx read node $outputDir."/deformed_0"
gfx read element $outputDir."/deformed"
# And the intial potential field
gfx read element $outputDir."/potential_0"

gfx set vis axes off

gfx create window 1 double_buffer
gfx modify window 1 image scene default light_model default
gfx modify window 1 image add_light default
gfx modify window 1 layout 2d ortho_axes z -y eye_spacing 0.25 width 680 height 619
gfx modify window 1 set current_pane 1
gfx modify window 1 background colour 0 0 0 texture none
gfx modify window 1 view parallel eye_point 13.6757 16.0611 92.3152 interest_point 13.6757 16.0611 0 up_vector 0 1 0 view_angle 31.99 near_clipping_plane 0.923152 far_clipping_plane 329.903 relative_viewport ndc_placement -1 1 2 2 viewport_coordinates 0 0 1 1
gfx modify window 1 overlay scene none
if ($movie || $images) {
		gfx modify window 1 set transform_tool current_pane 1 std_view_angle 40 normal_lines no_anti slow_transparency;
} else {
		gfx modify window 1 set transform_tool current_pane 1 std_view_angle 40 normal_lines no_anti fast_transparency;
}

if ($images) {
		gfx modify window 1 background colour 1 1 1 texture none;
		gfx modify window 1 set perturb_lines;
}

gfx create material white diff 1.0 1.0 1.0 
gfx create material blue diff 0.0 1.0 0.0 
gfx create material black diff 0.0 0.0 0.0 

##
## Use the deformed coordinates
##
gfx modify g_element sheet general clear circle_discretization 6 default_coordinate deformed element_discretization "20*20*20" native_discretization potential

##
## Draw the deformed and undeformed lines
##
gfx set line_width 2
if ($images) {
		gfx modify g_element sheet lines coordinate default_coordinate material blue;
} else {
		gfx modify g_element sheet lines coordinate default_coordinate material white;
}
gfx modify g_element sheet lines material black

##
## Create an appropriate spectrum
## Total Min. = -90, Total Max. =  50
gfx create spectrum potential log exaggeration 3 right reverse range -90 50 extend_above extend_below rainbow colour_range 0 1 ambient diffuse 

##
## Draw the potential surface
##
gfx modify g_element sheet general element_discretization '20*20*20' native_discretization potential
gfx modify g_element sheet surfaces data potential spectrum potential

##
## Set-up and draw the deforming fibres
##
# F = dx/dX
gfx define field F coordinate_system rectangular_cartesian gradient coordinate coordinates field deformed
# F(transpose)
gfx define field F_transpose coordinate_system rectangular_cartesian transpose source_number_of_rows 2 field F
# The deforming fibre orientation field
gfx define field fibre_axes coordinate_system rectangular_cartesian fibre_axes coordinate coordinates fibre fibres
gfx define field fibre_axes2 coordinate_system rectangular_cartesian composite fibre_axes.1 fibre_axes.2 fibre_axes.4 fibre_axes.5
gfx define field deformed_fibre_axes coordinate_system rectangular_cartesian matrix_multiply number_of_rows 2 fields fibre_axes2 F_transpose
gfx define field deformed_fibre_axis1 coordinate_system rectangular_cartesian composite deformed_fibre_axes.1 deformed_fibre_axes.2
# The element points showing the fibre orientation
gfx modify g_element sheet element_points glyph cylinder general size "0*0*0" centre 0.5,0,0 orientation deformed_fibre_axis1 scale_factors "3*0.15*0.15" use_elements cell_centres discretization "1*1*1" native_discretization NONE select_on material white selected_material default_selected

##
## Set-up and draw a representation of the principal strains
##
# C = F(transpose) . F
gfx define field C coordinate_system rectangular_cartesian matrix_multiply number_of_rows 2 fields F_transpose F
# 2D identity matrix
gfx define field identity2 coordinate_system rectangular_cartesian composite 1 0 0 1
# E = 0.5 (C - I)
gfx define field E2 coordinate_system rectangular_cartesian add fields C identity2 scale_factors 1 -1
gfx define field E coordinate_system rectangular_cartesian scale field E2 scale_factors 0.5 0.5 0.5 0.5
# Principal strains
gfx define field principal_strains coordinate_system rectangular_cartesian eigenvalues field E
gfx define field principal_strain1 coordinate_system rectangular_cartesian composite principal_strains.1
gfx define field principal_strain2 coordinate_system rectangular_cartesian composite principal_strains.2
# and the corresponding vectors
gfx define field principal_strain_vectors coordinate_system rectangular_cartesian eigenvectors eigenvalues principal_strains
gfx define field deformed_principal_strain_vectors coordinate_system rectangular_cartesian matrix_multiply number_of_rows 2 fields principal_strain_vectors F_transpose
gfx define field deformed_principal_strain_vector1 coordinate_system rectangular_cartesian composite deformed_principal_strain_vectors.1 deformed_principal_strain_vectors.2
gfx define field deformed_principal_strain_vector2 coordinate_system rectangular_cartesian composite deformed_principal_strain_vectors.3 deformed_principal_strain_vectors.4
gfx define field norm_def_principal_strain_vector1 coordinate_system rectangular_cartesian normalise field deformed_principal_strain_vector1
gfx define field norm_def_principal_strain_vector2 coordinate_system rectangular_cartesian normalise field deformed_principal_strain_vector2
# Create a spectrum for the strain
gfx create spectrum strain
gfx modify spectrum strain clear overwrite_colour
gfx modify spectrum strain linear reverse range -1 0 extend_below red colour_range 1 1 ambient diffuse component 1
gfx modify spectrum strain linear reverse range 0 1 extend_above blue colour_range 1 1 ambient diffuse component 1
gfx modify spectrum strain linear reverse range 0 1 extend_above green colour_range 0.5 0.5 ambient diffuse component 1
# Add some element points to show the principal strains
gfx modify g_element sheet element_points glyph mirror_cone general size "0*0*0" centre 0,0,0 orientation norm_def_principal_strain_vector1 variable_scale principal_strain1 scale_factors "10*0.4*0.4" use_elements cell_centres discretization "1*1*1" native_discretization NONE select_on material white data principal_strain1 spectrum strain selected_material default_selected
gfx modify g_element sheet element_points glyph mirror_cone general size "0*0*0" centre 0,0,0 orientation norm_def_principal_strain_vector2 variable_scale principal_strain2 scale_factors "10*0.4*0.4" use_elements cell_centres discretization "1*1*1" native_discretization NONE select_on material white data principal_strain2 spectrum strain selected_material default_selected

if ($movie) {
		gfx movie create test.mv quicktime window 1
}

for ($time=0;$time<=$tend;$time+=$tstep)
{
		print "Time = $time\n";
		gfx read node $outputDir."/deformed_$time"
		gfx read element $outputDir."/potential_$time"
		gfx update window 1
		if ($movie) {
				gfx movie force_onscreen add_frame
		}
		if ($images) {
				gfx print window 1 width 2048 height 2048 rgb file $imageDir/image_$time.rgb;
		}
}

if ($movie) {
		gfx movie end
}

