# Example showing how to use a cellml model

# If the example path is not set, default to current directory
if (! defined $example) {
    $example = "./";
}
# Drop off the trailing / in the example path
$chopped = chop $example;
if ($chopped ne "/") {
    $example .= $chopped;
}

fem define parameters;r;$example/cubic;

# default nodes, elements and grids to get a single grid point
fem define node;d;
fem define bas;d;
fem define elem;d;
fem define grid;d;
fem update grid geometry;
fem update grid metric;

# Define the a user defined CellML electrophysiological model
fem define equation;r;$example/cubic;

# Define the CellML model to use and map the parameters to CMISS and
# specify initial/boundary conditions for the cell model.
fem define cell;r;$example/cubic;

# Initialise the spatial properties of the model (there are none).
fem define material;r;$example/cubic cell;
fem define material;r;$example/cubic;
fem update grid material;

# Define the zero transmembrane flux boundary condition.
fem define initial;r;$example/cubic;

# Define an explicit Adams integrator.
fem define solve;r;$example/cubic;

# Initialise the solve.
fem solve to 0;

# Create a history file to store output variables of interest.
fem open history;cubic write variables yqs niqslist 1..3 binary;

# Loop through time integrating the cellular model.
$Tend = 150;
$dt = 1.0;
for ($time=0;$time<$Tend;$time+=$dt) {
    fem solve restart to $time;
    fem write history time $time variables yqs binary;
}

fem close history binary;

# Evaluate signals for the variables from the history file.
fem evaluate electrode;cubic history cubic from grid yqs iy 1..3 binary;

# Define an export of the CMISS signals to UnEMAP signals..
fem define export;r;$example/cubic;
# ..and export the signals to a UnEMAP signal file.
fem export signal;cubic electrodes signal cubic names cell;

# For testing we compare the generated CMISS signal files
if ($TESTING) {
    fem compare signal;cubic basis both masterfile $example/cubic comparefile cubic binary;
}
