# If the example path is not set, default to current directory
if (! defined $example) {
    $example = "./";
}
# Drop off the trailing / in the example path
$chopped = chop $example;
if ($chopped ne "/") {
    $example .= $chopped;
}

# We want to run the same model for multiple different parameter sets, so
# set up a function to do the solve and call it for each parameter set
sub runModel {
    $model = shift;

    fem define cell;r;"$example/$model";
    fem define material;d cell;
    fem define material;d;
    fem update grid material;
    fem define initial;d;
    fem define solve;r;$example/lsoda;
    fem solve to 0;
    fem open history;$model write variables yqs niqslist 1..13 binary;
    $Tend = 1500;
    $dt = 1.0;
    for ($time=0;$time<$Tend;$time+=$dt) {
        fem solve restart to $time;
        fem write history time $time variables yqs binary;
    }
    fem close history binary;
    fem evaluate electrode;$model history $model from grid yqs iy 1..13 binary;
    fem define export;r;$example/fenton_karma_model_1998;
    fem export signal;$model electrodes signal $model names cell;
}

fem define parameters;r;$example/cellml;
fem define node;d;
fem define bas;d;
fem define elem;d;
fem define grid;d;
fem update grid geometry;
fem update grid metric;
fem define equation;r;$example/cellml;

@models = ("FK_BR","FK_mod_BR","FK_mod_LR_I","FK_GP");
foreach $model (@models) {
    &runModel($model);
}

# For testing we compare the generated CMISS signal files
if ($TESTING) {
    foreach $model (@models) {
        fem compare signal;$model basis both masterfile "$example/$model" comparefile $model binary;
    }
}
