# Example showing how to use a cellml model

# Will hopefully soon be able to use the example variable in specifying
# the cellml model's URI in the ipcell file.

# If the example path is not set, default to current directory
if (! defined $example) {
    $example = "./";
}
# Drop off the trailing / in the example path
$chopped = chop $example;
if ($chopped ne "/") {
    $example .= $chopped;
}

fem define parameters;r;$example/hh;

# default nodes, elements and grids to get a single grid point
fem define node;d;
fem define bas;d;
fem define elem;d;
fem define grid;d;
fem update grid geometry;
fem update grid metric;

# Define the a user defined CellML electrophysiological model
fem define equation;r;$example/hh;

# Define the CellML model to use and map the parameters to CMISS and
# specify initial/boundary conditions for the cell model.
# Note: the "save temporary files" question is set to yes for testing
#       of the generated code. You might want to turn this off if you
#       are not running this as a test example.
fem define cell;r;$example/hh;

# Initialise the spatial properties of the model (there are none).
fem define material;r;$example/hh cell;
fem define material;r;$example/hh;
fem update grid material;

# Define the zero transmembrane flux boundary condition.
fem define initial;r;$example/hh;

# Define an explicit Adams integrator.
fem define solve;r;$example/hh;

# Initialise the solve.
fem solve to 0;

# Create a history file to store output variables of interest.
fem open history;hh write variables yqs niqslist 1..8 binary;

# Loop through time integrating the cellular model.
$Tend = 200;
$dt = 1.0;
for ($time=0;$time<$Tend;$time+=$dt) {
    fem solve restart to $time;
    fem write history time $time variables yqs binary;
}

fem close history binary;

# Evaluate signals for the variables from the history file.
fem evaluate electrode;hh history hh from gvariables yqs iy 1,2,3,4,5,6,7,8 electrodes 1 binary;

# Define an export of the CMISS signals to UnEMAP signals..
fem define export;r;$example/hh;

# ..and define names for each of the signals..
fem define leads;r;$example/hh;

# ..and export the signals to a UnEMAP signal file.
fem export signal;hh leads signal hh;

# For testing we compare the generated CMISS signal files
if ($TESTING) {
    fem compare signal;hh basis both masterfile $example/hh comparefile hh binary;
}
