# Example showing how to use a cellml model

# If the example path is not set, default to current directory
if (! defined $example) {
    $example = "./";
}
# Drop off the trailing / in the example path
$chopped = chop $example;
if ($chopped ne "/") {
    $example .= $chopped;
}

$format = "binary";

fem define parameters;r;$example/n98;

# default nodes, elements and grids to get a single grid point
fem define node;d;
fem define bas;d;
fem define elem;d;
fem define grid;d;
fem update grid geometry;
fem update grid metric;

# Define the a user defined CellML electrophysiological model
fem define equation;r;$example/n98;

# Define the CellML model to use and map the parameters to CMISS and
# specify initial/boundary conditions for the cell model.
fem define cell;r;$example/n98;

# Initialise the spatial properties of the model (there are none).
fem define material;r;$example/n98 cell;
fem define material;r;$example/n98;
fem update grid material;

# Define the zero transmembrane flux boundary condition.
fem define initial;r;$example/n98;

# Define an explicit Adams integrator.
fem define solve;r;$example/n98;

# Initialise the solve.
fem solve to 0;

# Create a history file to store output variables of interest.
fem open history;n98 write variables yqs niqslist 1..57 $format;
fem write history time 0 variables yqs $format;

# Loop through time integrating the cellular model.
$Tend = 500;
$dt = 1.0;
for ($time=$dt;$time<$Tend;$time+=$dt) {
    fem solve restart to $time;
    fem write history time $time variables yqs $format;
}

fem close history $format;

# Evaluate signals for the variables from the history file.
fem evaluate electrode;n98 history n98 from grid yqs iy 1..57 $format;

# Define an export of the CMISS signals to UnEMAP signals..
fem define export;r;$example/n98;
# ..and export the signals to a UnEMAP signal file.
fem export signal;n98 electrodes signal n98 names cell;

# For testing we compare the generated CMISS signal files
if ($TESTING) {
    fem compare signal;n98 basis both masterfile $example/n98 comparefile n98 $format;
}
