if( !$example ) {
  $example = ".";
}

# Set up two problem regions, one for the cylinder
# and one for the fibres
$tissue = 1;
$muscle = 2;

# Set up two problem classes - laplace for the cylinder
# and activation for the fibres 
$laplace = 1;
$active = 2;
 
# Read in the geometry of the two regions
fem de para;r;$example/test
fem define coordinates;r;$example/eccentric_dipole
fem define regi;r;$example/eccentric_dipole
fem define node;r;$example/eccentric_dipole reg $tissue
fem define node;r;$example/fibres reg $muscle
fem define bases;r;$example/eccentric_dipole
fem define elem;r;$example/eccentric_dipole reg $tissue
fem define elem;r;$example/fibres reg $muscle

# Group the cylinder nodes to allow boundary conditions
# to be easily set
fem group node 1 as top reg $tissue      
fem group node 2..13,15..58 as middle reg $tissue  
fem group node 14 as bottom reg $tissue  

# Generate a finite difference grid over the fibres
fem define grid;r;$example/fibres reg $muscle
fem update grid geometry reg $muscle
fem update grid metric reg $muscle

# Define the laplace problem in the cylinder
fem define equation;r;$example/eccentric_dipole reg $tissue class $laplace lock 
fem define material;r;$example/eccentric_dipole reg $tissue class $laplace
fem define initial;r;$example/eccentric_dipole reg $tissue class $laplace
fem define solve;r;$example/eccentric_dipole reg $tissue class $laplace

# Define the activation problem in the fibres
fem define equation;r;$example/fibres reg $muscle class $active
fem define material;r;$example/fibres reg $muscle class $active
fem define cell;r;$example/fibres reg $muscle class $active
fem update grid material reg $muscle class $active
fem define initial;r;$example/fibres reg $muscle class $active
fem define solve;r;$example/fibres reg $muscle class $active

# Perform an initial solve
fem solve reg $tissue class $laplace
fem solve to 0 reg $muscle class $active

# Loop over time
foreach $step ( 0..30 ) {
  $time = $step/2
  # Solve the grid problem
  fem solve restart to $time reg $muscle class $active
  # Calculate the dipole sources
  fem define sour;c grid 
  # Apply the sources to the laplace problem
  fem update sour reg $tissue class $laplace
  # Solve the laplace problem
  fem solve reg $tissue class $laplace
#  Uncomment to output the solution fields over time
#    fem export elem;time$time reg $muscle class $active field as test2
#    fem export node;bemt$time reg $tissue class $laplace field as test
}

#fem export node;exfiles/test as test
#fem export elem;exfiles/test as test
#fem export elem;exfiles/bemt reg $tissue class $laplace field as test 
#fem export node;exfiles/test2 as test2 reg 2
#fem export elem;exfiles/test2 as test2 reg 2
