# Example e41
# -----------
#


#----------------------------------------------------------#
#
# Customising the Full Torso
# This example uses the full 'generic' torso model and 
# customises each region using measurements taken from a 
# volunteer to give a better model for that individual. 
# This example is not run nightly as some of the 
# optimisation steps take a while to run.
#
#----------------------------------------------------------#


fem def para;r;full_body;example             #Read in parameter file.
fem def coor 3,1                             #Define 3-d coordinate system.
fem def base;r;;example                      #Define bicubic Hermite and sector 
#                                            # basis functions.
fem def regi;r;;example                      #Define six regions for full torso.
fem def node;r;;example region all           #Read in nodes for all regions.
fem def elem;r;;example region all           #Read in elements for all regions.
fem def fiel;r;;example region all           #Read in field of geometry.
fem def elem;r;;example field region all     #Read in element field.
fem update field from geometry region all    #Update field from initial torso 
#                                            # geometry.


#
#----------------------------#
# Customise outer layer
#----------------------------#
#
fem def opti;r;outer_torso;example           #Read in optimisation parameters to 
#                                            # customise the outer surface. This 
#                                            # file specifies the number of 
#                                            # polynomial and cosine coefficients 
#                                            # to be used. Also defines torso 
#                                            # lengths and circumferential
#                                            # measurements from the patient torso.  
#
optimise                                     #Carries out NPSOL non-linear 
#                                            # optimisation. Optimisation takes
#                                            # about 10 minutes.
#
fem update field from geometry               #Updates the field to incorporate customised
#                                            # outer surface.
#


#
#----------------------------#
# Customise fat layer
#  This takes ages (> 3 hrs) 
#  so remove for now 
#----------------------------#
#
fem def opti;r;fat_layer;example             #Read in optimisation parameters to 
#                                            # customise the fat layer. Specifies 
#                                            # the number of coefficients to be 
#                                            # used and the desired percentage 
#                                            # body fat in terms of volume not 
#                                            # mass.
#
#optimise                                    #Carries out NPSOL non-linear 
#                                            # optimisation. Optimisation takes 
#                                            # about an hour because there are a
#                                            # large number of polynomial terms
#                                            # used to customise it and at each
#                                            # step each node in the fat-muscle 
#                                            # surface is checked to make sure its 
#                                            # still inside the outer torso surface.
#
fem update field from geometry               #Updates the field to incorporate 
#                                            # customised fat-muscle interface 
#                                            # surface.
#
fem def cust;r;muscle_layer;example          #Read in a customisation parameters 
#                                            # for muscle layer. Same length 
#                                            # scaling as for fat layer and the 
#                                            # first polynomial coeff which is 
#                                            # just a radial scaling from the 
#                                            # fat layer customisation is used 
#                                            # to scale.
fem change mesh region 3                     #Applies the customisation parameters 
#                                            # to the torso cavity surace region 3.
#
fem update field from geometry               #Updates the field to incorporate 
#                                            # customised torso cavity surface.


#
#                                            #The heart surface was customised 
#                                            # by eye by comparison with 
#                                            # digitised point of the heart and
#                                            # body surface from 3 orthogonal
#                                            # MRI images. The lungs were each 
#                                            # grouped into two regions. The 
#                                            # region surrounding the heart or
#                                            # cardiac notch and the rest of the
#                                            # lung. The cardiac notch region of
#                                            # each lung is moved in the same 
#                                            # way as the heart to maintain 
#                                            # correct spacing between the 
#                                            # surfaces. The outer lung surfaces
#                                            # were scaled by same factor as fat 
#                                            # and muscle.


# Grouping specific nodes
#
fem group nodes 919..955 as heart region 6   #Grouping nodes for heart and lungs. 
fem group nodes 854,858,862,867..870,875..878,883..885 as rlnotch region 5
fem group nodes 768..772,776..780,784..788,792..796,800..804 as llnotch reg 4
fem group nodes 763..767,773..775,781..783,789..791,797..799,805..840 as llout reg 4
fem group nodes 841..853,855..857,859..861,863..866,871..874,879..882,886..918 as rlout reg 5


#
#----------------------------#
# Rotate heart and lung 
#  cardiac notch nodes
#  according to MRI data
#----------------------------#
#
fem change nodes rotate by -4 axis 1,0,0 no heart reg all
fem change nodes rotate by -4 axis 1,0,0 no rlnotch reg all
fem change nodes rotate by -4 axis 1,0,0 no llnotch reg all
fem change nodes rotate by -4 axis 0,1,0 no heart reg all
fem change nodes rotate by -4 axis 0,1,0 no rlnotch reg all
fem change nodes rotate by -4 axis 0,1,0 no llnotch reg all


#
#----------------------------#
# Translate heart and cardiac 
#  notch nodes
#----------------------------#
#
fem change nodes translate by 2,1,-4 no heart reg all
fem change nodes translate by 2,1,-4 no rlnotch reg all
fem change nodes translate by 2,1,-4 no llnotch reg all


#
#----------------------------#
# Scale heart and cardiac 
#  notch node
#----------------------------#
#
fem change nodes scale by 1,1,1.05 no heart reg all 
fem change nodes scale by 1,1,1.05 no rlnotch reg all
fem change nodes scale by 1,1,1.05 no llnotch reg all 


#
#----------------------------#
# Scale outer lung nodes
#----------------------------#
#
fem change nodes scale by 0.96,0.96,0.96 no rlout reg all
fem change nodes scale by 0.96,0.96,0.96 no llout reg all


#
#----------------------------#
# Export geometry
#----------------------------#
#
fem export nodes;full_body reg all            
