#!/usr/bin/perl -w
#

use strict;

my $xi1;
my $xi2;
my $xi3;
my $inputFileName;
my @Potentials = ();

#=============================================================================

sub ParseInputExelemFileWriteShape{

  ( $xi1, $xi2, $xi3, $inputFileName, @Potentials ) = @_;
  my $FoundValues = 0;
  my $FoundNodes = 0;
  my $BeforeFields = 1;
  my $BeforeValues = 0;
  my @InputLine;

  CORE::open( INPUTFILE, "$inputFileName".".exelem" )    or die ( "Unable to open input file\n" );
  CORE::open( OUTPUTFILE, ">$inputFileName"."SHAPE.exelem" ) or die ( "Unable to open output file\n" );
  print( " Parsing input file... \n" );
  while( <INPUTFILE> ) {
    @InputLine= split;

    if( /Fields=/ ) {
      $BeforeFields = 0;
      print( OUTPUTFILE '#Fields=1'."\n" );
      print( OUTPUTFILE ' 1) shape, field, rectangular cartesian, #Components=1'."\n" );
      print( OUTPUTFILE '   shape.  l.Lagrange*l.Lagrange*l.Lagrange, no modify, grid based.'."\n" );
    }
 
    if( $BeforeFields ) {
      print( OUTPUTFILE $_ );
    }

    if( /xi1=/ && /xi2=/ ) {
      $BeforeValues = 1;
      $xi1 = substr( $InputLine[1], 0, -1 ) + 1;
      $xi2 = substr( $InputLine[3], 0, -1 ) + 1;
      $xi3 = $InputLine[5] + 1;
    }

    if( /Nodes:/ ) {
      $FoundValues = 0;
      $FoundNodes = 1;
    }

    if( $FoundValues ) {
      print( OUTPUTFILE "     " );
      foreach ( split ) {
        if ( /0.00000E\+00/ ) {
          push @Potentials, "nan";
          print( OUTPUTFILE " 0.10000E+01" );
        } else {
          print( OUTPUTFILE " 0.00000E+00" );
          push @Potentials, $_;
        }
      }
      print( OUTPUTFILE "\n" );
    }

    if( $FoundNodes ) {
      print( OUTPUTFILE $_ );
    }

    if( $BeforeValues ) {
      print( OUTPUTFILE $_ );
    }

    if( /Values:/ ) {
      $BeforeValues = 0;
       $FoundValues = 1;
    }
  }
  CORE::close( INPUTFILE );
  CORE::close( OUTPUTFILE );
  print( " Writing shape file... \n" );

} # ParseInputExelemFileWriteShape

#=============================================================================

sub BiLinearExpansion1pt {

  ( $xi1, $xi2, $xi3, @Potentials ) = @_;
  my $FOUND = 1;
  my $count = 0;

  while( $FOUND && ($count <= 200) ) {
    $FOUND = 0;
    $count++;
    my $counter = 0;
    my $counter2 = 0;
    print( " Performing pass $count" );
    foreach my $nq3 ( 1..$xi3 ) {
      foreach my $nq2 ( 1..$xi2 ) {
        foreach my $nq1 ( 1..$xi1 ) {
          my $nq = $nq1 + (($nq2-1)*$xi1) + (($nq3-1)*$xi1*$xi2);
          if( $Potentials[$nq-1] eq "nan" ) {
            my @extremes;
            if( $nq1 == 1 ) {
              $extremes[0] = 0;
            } else {
              $extremes[0] = $nq - 1;
            }
            if( $nq2 == 1 ) {
              $extremes[1] = 0;
            } else {
              $extremes[1] = $nq - $xi1;
            }
            if( $nq3 == 1 ) {
              $extremes[2] = 0;
            } else {
              $extremes[2] = $nq - ($xi1*$xi2);
            }
            if( $nq1 == $xi1 ) {
              $extremes[3] = 0;
            } else {
              $extremes[3] = $nq + 1;
            }
            if( $nq2 == $xi2 ) {
              $extremes[4] = 0;
            } else {
              $extremes[4] = $nq + $xi1;
            }
            if( $nq3 == $xi3 ) {
              $extremes[5] = 0;
            } else {
              $extremes[5] = $nq + ($xi1*$xi2);
            }
          
            my $hasValue = 0;
            my $averageValue = 0;
            foreach my $i ( 1..@extremes ) {
              if( $extremes[$i-1] ) {
                if( $Potentials[$extremes[$i-1]-1] ne "nan" ) {
                  $hasValue++;
                  $averageValue += $Potentials[$extremes[$i-1]-1];
                }
              }
            }
            if( $hasValue ) {
              $Potentials[$nq-1] = $averageValue / $hasValue;
              $counter++;
            } else {
              $FOUND = 1;
              $counter2++;
            }
          }
        }
      }
    }
    print( ", $counter records updated, $counter2 are outstanding\n" );
  }

} # BiLinearExpansion1pt

#=============================================================================

sub WritePoteExelemFile {

  ( $xi1, $xi2, $xi3, $inputFileName, @Potentials ) = @_;
  my $BeforeValues = 1;
  my $FoundNodes = 0;
  my $firstTime = 1;

  CORE::open( INPUTFILE, "$inputFileName".".exelem" )    or die ( "Unable to open input file\n" );
  CORE::open( OUTPUTFILE, ">$inputFileName"."POTEN.exelem" ) or die ( "Unable to open output file\n" );

  print( " Writing output file... \n" );
  while( <INPUTFILE> ) {
    if( $BeforeValues ) {
      print( OUTPUTFILE $_ );
    } 
    if( /Values:/ ) {
      $BeforeValues = 0;
    }

    if( /Nodes:/ ) {
      $FoundNodes = 1;
    }
    if( $FoundNodes ) {
      print( OUTPUTFILE $_ );
    }

    if( $FoundNodes == 0 && $BeforeValues == 0 && $firstTime ) {
      foreach my $i ( 1..@Potentials ) {
        print( OUTPUTFILE "  $Potentials[$i-1]" );
        if( $i%5 == 0 ) {
          print( OUTPUTFILE "\n" );
        }      
      }
      if( @Potentials%5 != 0 ) {
        print( OUTPUTFILE "\n" );
      }
      $firstTime = 0;
    }
  }

  CORE::close( INPUTFILE );
  CORE::close( OUTPUTFILE );

} # WritePoteExelemFile

#=============================================================================

# Main Program
# ============

# Get file name with or without exelem extension
#print( " Enter the potential field file name: " );
#$inputFileName = <STDIN>;
#chomp( $inputFileName );
#if( substr( $inputFileName, -7, 7 ) eq '.exelem' ) {
#  $inputFileName = substr( $inputFileName, 0, -7 );
#}

# Hard code file name for testing
$inputFileName = 'spote';

# Get the potentials and write the shape file
&ParseInputExelemFileWriteShape( $xi1, $xi2, $xi3, $inputFileName, @Potentials );

# Expand the potential field (1 adjacent point)
&BiLinearExpansion1pt( $xi1, $xi2, $xi3, @Potentials );

# Write out the expanded potential field
&WritePoteExelemFile( $xi1, $xi2, $xi3, $inputFileName, @Potentials );
