
sub WriteGradPhiExelem {
  # This Perl subroutine is used to combine three potentential gradient
  # exelem files into a single three component field within a single
  # exelem file. This routine requires filenames for the three input
  # exelems to be passed and also a filename for the output exelem.

  ( $DXFILENAME, $DYFILENAME, $DZFILENAME, $OUTPUTNAME ) = @_;

  my $Found = 0;
  my $Found2 = 0;
  my $First = 1;
  my $InputLine = ' ';

  CORE::open( OUTPUTFILE,">$OUTPUTNAME" ) or die( "ERROR: Unable to open output file\n" );
  CORE::open( INPUTFILE,"<$DXFILENAME" ) or die( "ERROR: Unable to open DX input file\n" );

  while( <INPUTFILE> ) {
    if( /Nodes:/ ) {
      $Inputline = $_;
      $Found2 = 0;

      CORE::open( INPUTFILE2,"<$DYFILENAME" ) or die( "ERROR: Unable to open DY input file\n" );
      while( <INPUTFILE2> ) {
        if( /Nodes:/ ) {
          $Found2 = 0;
        }
        if( $Found2 ) {
          print( OUTPUTFILE "$_" );
        }
        if( /Values:/ ) {
          $Found2 = 1;
        }
      }
      CORE::close( INPUTFILE2 );

      $Found2 = 0;
      CORE::open( INPUTFILE2,"<$DZFILENAME" ) or die( "ERROR: Unable to open DZ input file\n" );
      while( <INPUTFILE2> ) {
        if( /Nodes:/ ) {
          $Found2 = 0;
        }
        if( $Found2 ) {
          print( OUTPUTFILE "$_" );
        }
        if( /Values:/ ) {
          $Found2 = 1;
        }
      }
      CORE::close( INPUTFILE2 );

      print( OUTPUTFILE "$Inputline" );
    }

    if( !$Found ) {
      print( OUTPUTFILE $_ );
    } elsif( $First ) {
      print( OUTPUTFILE ' 1) H, field, rectangular cartesian, #Components=3'."\n");
      print( OUTPUTFILE '   x.  l.Lagrange*l.Lagrange*l.Lagrange, no modify, grid based.'."\n");
      $First = 0;
    } 
    if( /Fields/ ) {
      $Found = 1;
    }
    if( /xi1/ ) {
      print( OUTPUTFILE $_ );
      print( OUTPUTFILE '   y.  l.Lagrange*l.Lagrange*l.Lagrange, no modify, grid based.'."\n");
      print( OUTPUTFILE $_ );
      print( OUTPUTFILE '   z.  l.Lagrange*l.Lagrange*l.Lagrange, no modify, grid based.'."\n");
      print( OUTPUTFILE $_ );
    }
    if(( /Element/ ) && $Found ) {
      print( OUTPUTFILE $_ );
      $Found = 0;     
    }
  }
  CORE::close( INPUTFILE );
  CORE::close( OUTPUTFILE );
       
} # WriteGradPhiExelem

